@@ -64,6 +64,7 @@
 #include "util.h"
 #include "LogFactory.h"
 #include "DownloadContext.h"
+#include "wallclock.h"
 
 namespace aria2 {
 
@@ -97,13 +98,23 @@ AbstractCommand::~AbstractCommand() {
 }
 
 bool AbstractCommand::execute() {
-  logger->debug("CUID#%d - socket: read:%d, write:%d, hup:%d, err:%d",
-                cuid, _readEvent, _writeEvent, _hupEvent, _errorEvent);
+  if(logger->debug()) {
+    logger->debug("CUID#%d - socket: read:%d, write:%d, hup:%d, err:%d",
+                  cuid, _readEvent, _writeEvent, _hupEvent, _errorEvent);
+  }
   try {
     if(_requestGroup->downloadFinished() || _requestGroup->isHaltRequested()) {
       //logger->debug("CUID#%d - finished.", cuid);
       return true;
     }
+    if(!req.isNull() && req->removalRequested()) {
+      if(logger->debug()) {
+        logger->debug
+          ("CUID#%d - Discard original URI=%s because it is requested.",
+           cuid, req->getUri().c_str());
+      }
+      return prepareForRetry(0);
+    }
     // TODO it is not needed to check other PeerStats every time.
     // Find faster Request when no segment is available.
     if(!req.isNull() && _fileEntry->countPooledRequest() > 0 &&
@@ -132,7 +143,7 @@ bool AbstractCommand::execute() {
        (nameResolverCheck && nameResolveFinished()) ||
 #endif // ENABLE_ASYNC_DNS
        (!checkSocketIsReadable && !checkSocketIsWritable && !nameResolverCheck)) {
-      checkPoint.reset();
+      checkPoint = global::wallclock;
       if(!_requestGroup->getPieceStorage().isNull()) {
         _segments.clear();
         _requestGroup->getSegmentMan()->getInFlightSegment(_segments, cuid);
@@ -151,6 +162,9 @@ bool AbstractCommand::execute() {
             // When all segments are ignored in SegmentMan, there are
             // no URIs available, so don't retry.
             if(_requestGroup->getSegmentMan()->allSegmentsIgnored()) {
+              if(logger->debug()) {
+                logger->debug("All segments are ignored.");
+              }
               return true;
             } else {
               return prepareForRetry(1);
@@ -173,7 +187,7 @@ bool AbstractCommand::execute() {
         (StringFormat(MSG_NETWORK_PROBLEM,
                       socket->getSocketError().c_str()).str());
     } else {
-      if(checkPoint.elapsed(timeout)) {
+      if(checkPoint.difference(global::wallclock) >= timeout) {
         // timeout triggers ServerStat error state.
 
         SharedHandle<ServerStat> ss =
@@ -188,14 +202,16 @@ bool AbstractCommand::execute() {
     }
   } catch(DlAbortEx& err) {
     if(req.isNull()) {
-      logger->debug(EX_EXCEPTION_CAUGHT, err);
+      if(logger->debug()) {
+        logger->debug(EX_EXCEPTION_CAUGHT, err);
+      }
     } else {
       logger->error(MSG_DOWNLOAD_ABORTED,
                     DL_ABORT_EX2(StringFormat
-                                 ("URI=%s", req->getCurrentUrl().c_str()).str(),err),
-                    cuid, req->getUrl().c_str());
-      _fileEntry->addURIResult(req->getUrl(), err.getCode());
-      _requestGroup->setLastUriResult(req->getUrl(), err.getCode());
+                                 ("URI=%s", req->getCurrentUri().c_str()).str(),err),
+                    cuid, req->getUri().c_str());
+      _fileEntry->addURIResult(req->getUri(), err.getCode());
+      _requestGroup->setLastUriResult(req->getUri(), err.getCode());
     }
     onAbort();
     tryReserved();
@@ -204,8 +220,8 @@ bool AbstractCommand::execute() {
     assert(!req.isNull());
     logger->info(MSG_RESTARTING_DOWNLOAD,
                  DL_RETRY_EX2(StringFormat
-                              ("URI=%s", req->getCurrentUrl().c_str()).str(),err),
-                 cuid, req->getUrl().c_str());
+                              ("URI=%s", req->getCurrentUri().c_str()).str(),err),
+                 cuid, req->getUri().c_str());
     req->addTryCount();
     req->resetRedirectCount();
     const unsigned int maxTries = getOption()->getAsInt(PREF_MAX_TRIES);
@@ -213,9 +229,9 @@ bool AbstractCommand::execute() {
     if(isAbort) {
       onAbort();
       logger->info(MSG_MAX_TRY, cuid, req->getTryCount());
-      logger->error(MSG_DOWNLOAD_ABORTED, err, cuid, req->getUrl().c_str());
-      _fileEntry->addURIResult(req->getUrl(), err.getCode());
-      _requestGroup->setLastUriResult(req->getUrl(), err.getCode());
+      logger->error(MSG_DOWNLOAD_ABORTED, err, cuid, req->getUri().c_str());
+      _fileEntry->addURIResult(req->getUri(), err.getCode());
+      _requestGroup->setLastUriResult(req->getUri(), err.getCode());
       tryReserved();
       return true;
     } else {
@@ -224,8 +240,8 @@ bool AbstractCommand::execute() {
   } catch(DownloadFailureException& err) {
     logger->error(EX_EXCEPTION_CAUGHT, err);
     if(!req.isNull()) {
-      _fileEntry->addURIResult(req->getUrl(), err.getCode());
-      _requestGroup->setLastUriResult(req->getUrl(), err.getCode());
+      _fileEntry->addURIResult(req->getUri(), err.getCode());
+      _requestGroup->setLastUriResult(req->getUri(), err.getCode());
     }
     _requestGroup->setHaltRequested(true);
     return true;
@@ -240,14 +256,18 @@ void AbstractCommand::tryReserved() {
     // and there are no URI left. Because file length is unknown, we
     // can assume that there are no in-flight request object.
     if(entry->getLength() == 0 && entry->getRemainingUris().empty()) {
-      logger->debug("CUID#%d - Not trying next request."
-                    " No reserved/pooled request is remaining and"
-                    " total length is still unknown.", cuid);
+      if(logger->debug()) {
+        logger->debug("CUID#%d - Not trying next request."
+                      " No reserved/pooled request is remaining and"
+                      " total length is still unknown.", cuid);
+      }
       return;
     }
   }
-  logger->debug("CUID#%d - Trying reserved/pooled request.", cuid);
-  Commands commands;
+  if(logger->debug()) {
+    logger->debug("CUID#%d - Trying reserved/pooled request.", cuid);
+  }
+  std::vector<Command*> commands;
   _requestGroup->createNextCommand(commands, e, 1);
   e->setNoWait(true);
   e->addCommand(commands);
@@ -259,8 +279,10 @@ bool AbstractCommand::prepareForRetry(time_t wait) {
   }
   if(!req.isNull()) {
     _fileEntry->poolRequest(req);
-    logger->debug("CUID#%d - Pooling request URI=%s",
-                  cuid, req->getUrl().c_str());
+    if(logger->debug()) {
+      logger->debug("CUID#%d - Pooling request URI=%s",
+                    cuid, req->getUri().c_str());
+    }
     if(!_requestGroup->getSegmentMan().isNull()) {
       _requestGroup->getSegmentMan()->recognizeSegmentFor(_fileEntry);
     }
@@ -283,11 +305,12 @@ void AbstractCommand::onAbort() {
     // TODO This might be a problem if the failure is caused by proxy.
     e->_requestGroupMan->getOrCreateServerStat(req->getHost(),
                                                req->getProtocol())->setError();
-    _fileEntry->removeIdenticalURI(req->getUrl());
+    _fileEntry->removeIdenticalURI(req->getUri());
     _fileEntry->removeRequest(req);
   }
-
-  logger->debug("CUID#%d - Aborting download", cuid);
+  if(logger->debug()) {
+    logger->debug("CUID#%d - Aborting download", cuid);
+  }
   if(!_requestGroup->getPieceStorage().isNull()) {
     _requestGroup->getSegmentMan()->cancelSegment(cuid);
   }
@@ -398,7 +421,7 @@ static bool isProxyRequest
 (const std::string& protocol, const SharedHandle<Option>& option)
 {
   const std::string& proxyUri = getProxyUri(protocol, option);
-  return !proxyUri.empty() && Request().setUrl(proxyUri);
+  return !proxyUri.empty() && Request().setUri(proxyUri);
 }
 
 class DomainMatch {
@@ -445,10 +468,14 @@ SharedHandle<Request> AbstractCommand::createProxyRequest() const
   std::string proxy = getProxyUri(req->getProtocol(), getOption());
   if(!proxy.empty()) {
     proxyRequest.reset(new Request());
-    if(proxyRequest->setUrl(proxy)) {
-      logger->debug("CUID#%d - Using proxy", cuid);      
+    if(proxyRequest->setUri(proxy)) {
+      if(logger->debug()) {
+        logger->debug("CUID#%d - Using proxy", cuid);
+      }
     } else {
-      logger->debug("CUID#%d - Failed to parse proxy string", cuid);
+      if(logger->debug()) {
+        logger->debug("CUID#%d - Failed to parse proxy string", cuid);
+      }
       proxyRequest.reset();
     }
   }
@@ -488,7 +515,7 @@ bool AbstractCommand::asyncResolveHostname()
   }
 }
 
-const std::deque<std::string>& AbstractCommand::getResolvedAddresses()
+const std::vector<std::string>& AbstractCommand::getResolvedAddresses()
 {
   return _asyncNameResolver->getResolvedAddresses();
 }
@@ -518,10 +545,10 @@ bool AbstractCommand::nameResolveFinished() const {
 
 void AbstractCommand::prepareForNextAction(Command* nextCommand)
 {
-  CheckIntegrityEntryHandle entry
+  SharedHandle<CheckIntegrityEntry> entry
     (new StreamCheckIntegrityEntry(_requestGroup, nextCommand));
 
-  std::deque<Command*> commands;
+  std::vector<Command*> commands;
   try {
     _requestGroup->processCheckIntegrityEntry(commands, entry, e);
   } catch(RecoverableException& e) {
