@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -35,6 +35,7 @@
 #include "DefaultBtInteractive.h"
 
 #include <cstring>
+#include <vector>
 
 #include "prefs.h"
 #include "message.h"
@@ -69,6 +70,9 @@
 #include "RequestGroup.h"
 #include "RequestGroupMan.h"
 #include "bittorrent_helper.h"
+#include "UTMetadataRequestFactory.h"
+#include "UTMetadataRequestTracker.h"
+#include "wallclock.h"
 
 namespace aria2 {
 
@@ -78,12 +82,14 @@ DefaultBtInteractive::DefaultBtInteractive
   :
   _downloadContext(downloadContext),
   peer(peer),
+  _metadataGetMode(false),
   logger(LogFactory::getInstance()),
   allowedFastSetSize(10),
   keepAliveInterval(120),
   _utPexEnabled(false),
   _dhtEnabled(false),
-  _numReceivedMessage(0)
+  _numReceivedMessage(0),
+  _maxOutstandingRequest(DEFAULT_MAX_OUTSTANDING_REQUEST)
 {}
 
 DefaultBtInteractive::~DefaultBtInteractive() {}
@@ -103,7 +109,7 @@ BtMessageHandle DefaultBtInteractive::receiveHandshake(bool quickReply) {
     return SharedHandle<BtMessage>();
   }
   if(memcmp(message->getPeerId(), bittorrent::getStaticPeerId(),
-	    PEER_ID_LENGTH) == 0) {
+            PEER_ID_LENGTH) == 0) {
     throw DL_ABORT_EX
       (StringFormat
        ("CUID#%d - Drop connection from the same Peer ID", cuid).str());
@@ -127,8 +133,8 @@ BtMessageHandle DefaultBtInteractive::receiveHandshake(bool quickReply) {
     logger->info(MSG_DHT_ENABLED_PEER, cuid);
   }
   logger->info(MSG_RECEIVE_PEER_MESSAGE, cuid,
-	       peer->ipaddr.c_str(), peer->port,
-	       message->toString().c_str());
+               peer->ipaddr.c_str(), peer->port,
+               message->toString().c_str());
   return message;
 }
 
@@ -139,17 +145,21 @@ BtMessageHandle DefaultBtInteractive::receiveAndSendHandshake() {
 void DefaultBtInteractive::doPostHandshakeProcessing() {
   // Set time 0 to haveCheckPoint to cache http/ftp download piece completion
   haveCheckPoint.setTimeInSec(0);
-  keepAliveCheckPoint.reset();
-  floodingCheckPoint.reset();
+  keepAliveCheckPoint = global::wallclock;
+  floodingCheckPoint = global::wallclock;
   _pexCheckPoint.setTimeInSec(0);
   if(peer->isExtendedMessagingEnabled()) {
     addHandshakeExtendedMessageToQueue();
   }
-  addBitfieldMessageToQueue();
+  if(!_metadataGetMode) {
+    addBitfieldMessageToQueue();
+  }
   if(peer->isDHTEnabled() && _dhtEnabled) {
     addPortMessageToQueue();
   }
-  addAllowedFastMessageToQueue();  
+  if(!_metadataGetMode) {
+    addAllowedFastMessageToQueue();
+  }
   sendPendingMessage();
 }
 
@@ -166,7 +176,10 @@ void DefaultBtInteractive::addHandshakeExtendedMessageToQueue()
   m->setClientVersion(CLIENT_ARIA2);
   m->setTCPPort(_btRuntime->getListenPort());
   m->setExtensions(_extensionMessageRegistry->getExtensions());
-  
+  const BDE& attrs = _downloadContext->getAttribute(bittorrent::BITTORRENT);
+  if(attrs.containsKey(bittorrent::METADATA)) {
+    m->setMetadataSize(attrs[bittorrent::METADATA_SIZE].i());
+  }
   SharedHandle<BtMessage> msg = messageFactory->createBtExtendedMessage(m);
   dispatcher->addMessageToQueue(msg);
 }
@@ -191,13 +204,13 @@ void DefaultBtInteractive::addAllowedFastMessageToQueue() {
   if(peer->isFastExtensionEnabled()) {
     std::vector<size_t> fastSet;
     bittorrent::computeFastSet(fastSet, peer->ipaddr,
-			       _downloadContext->getNumPieces(),
-			       bittorrent::getInfoHash(_downloadContext),
-			       allowedFastSetSize);
-    for(std::vector<size_t>::const_iterator itr = fastSet.begin();
-	itr != fastSet.end(); ++itr) {
+                               _downloadContext->getNumPieces(),
+                               bittorrent::getInfoHash(_downloadContext),
+                               allowedFastSetSize);
+    for(std::vector<size_t>::const_iterator itr = fastSet.begin(),
+          eoi = fastSet.end(); itr != eoi; ++itr) {
       dispatcher->addMessageToQueue
-	(messageFactory->createAllowedFastMessage(*itr));
+        (messageFactory->createAllowedFastMessage(*itr));
     }
   }
 }
@@ -215,9 +228,9 @@ void DefaultBtInteractive::decideChoking() {
 }
 
 void DefaultBtInteractive::checkHave() {
-  std::deque<size_t> indexes;
+  std::vector<size_t> indexes;
   _pieceStorage->getAdvertisedPieceIndexes(indexes, cuid, haveCheckPoint);
-  haveCheckPoint.reset();
+  haveCheckPoint = global::wallclock;
   if(indexes.size() >= 20) {
     if(peer->isFastExtensionEnabled() && _pieceStorage->allDownloadFinished()) {
       dispatcher->addMessageToQueue(messageFactory->createHaveAllMessage());
@@ -225,22 +238,23 @@ void DefaultBtInteractive::checkHave() {
       dispatcher->addMessageToQueue(messageFactory->createBitfieldMessage());
     }
   } else {
-    for(std::deque<size_t>::iterator itr = indexes.begin();
-	itr != indexes.end(); ++itr) {
+    for(std::vector<size_t>::const_iterator itr = indexes.begin(),
+          eoi = indexes.end(); itr != eoi; ++itr) {
       dispatcher->addMessageToQueue(messageFactory->createHaveMessage(*itr));
     }
   }
 }
 
 void DefaultBtInteractive::sendKeepAlive() {
-  if(keepAliveCheckPoint.elapsed(keepAliveInterval)) {
+  if(keepAliveCheckPoint.difference(global::wallclock) >= keepAliveInterval) {
     dispatcher->addMessageToQueue(messageFactory->createKeepAliveMessage());
     dispatcher->sendMessages();
-    keepAliveCheckPoint.reset();
+    keepAliveCheckPoint = global::wallclock;
   }
 }
 
 size_t DefaultBtInteractive::receiveMessages() {
+  size_t countOldOutstandingRequest = dispatcher->countOutstandingRequest();
   size_t msgcount = 0;
   for(int i = 0; i < 50; ++i) {
     if(_requestGroupMan->doesOverallDownloadSpeedExceed() ||
@@ -252,9 +266,11 @@ size_t DefaultBtInteractive::receiveMessages() {
       break;
     }
     ++msgcount;
-    logger->info(MSG_RECEIVE_PEER_MESSAGE, cuid,
-		 peer->ipaddr.c_str(), peer->port,
-		 message->toString().c_str());
+    if(logger->info()) {
+      logger->info(MSG_RECEIVE_PEER_MESSAGE, cuid,
+                   peer->ipaddr.c_str(), peer->port,
+                   message->toString().c_str());
+    }
     message->doReceivedAction();
 
     switch(message->getId()) {
@@ -263,37 +279,46 @@ size_t DefaultBtInteractive::receiveMessages() {
       break;
     case BtChokeMessage::ID:
       if(!peer->peerChoking()) {
-	floodingStat.incChokeUnchokeCount();
+        floodingStat.incChokeUnchokeCount();
       }
       break;
     case BtUnchokeMessage::ID:
       if(peer->peerChoking()) {
-	floodingStat.incChokeUnchokeCount();
+        floodingStat.incChokeUnchokeCount();
       }
       break;
     case BtPieceMessage::ID:
       _peerStorage->updateTransferStatFor(peer);
       // pass through
     case BtRequestMessage::ID:
-      inactiveCheckPoint.reset();
+      inactiveCheckPoint = global::wallclock;
       break;
     }
   }
+  if(countOldOutstandingRequest > 0 &&
+     dispatcher->countOutstandingRequest() == 0){
+    _maxOutstandingRequest = std::min((size_t)UB_MAX_OUTSTANDING_REQUEST,
+                                      _maxOutstandingRequest*2);
+  }
   return msgcount;
 }
 
 void DefaultBtInteractive::decideInterest() {
   if(_pieceStorage->hasMissingPiece(peer)) {
     if(!peer->amInterested()) {
-      logger->debug(MSG_PEER_INTERESTED, cuid);
+      if(logger->debug()) {
+        logger->debug(MSG_PEER_INTERESTED, cuid);
+      }
       dispatcher->
-	addMessageToQueue(messageFactory->createInterestedMessage());
+        addMessageToQueue(messageFactory->createInterestedMessage());
     }
   } else {
     if(peer->amInterested()) {
-      logger->debug(MSG_PEER_NOT_INTERESTED, cuid);
+      if(logger->debug()) {
+        logger->debug(MSG_PEER_NOT_INTERESTED, cuid);
+      }
       dispatcher->
-	addMessageToQueue(messageFactory->createNotInterestedMessage());
+        addMessageToQueue(messageFactory->createNotInterestedMessage());
     }
   }
 }
@@ -305,56 +330,50 @@ void DefaultBtInteractive::fillPiece(size_t maxMissingBlock) {
 
     if(peer->peerChoking()) {
       if(peer->isFastExtensionEnabled()) {
-	std::deque<size_t> excludedIndexes;
-	btRequestFactory->getTargetPieceIndexes(excludedIndexes);
-	while(numMissingBlock < maxMissingBlock) {
-	  SharedHandle<Piece> piece =
-	    _pieceStorage->getMissingFastPiece(peer, excludedIndexes);
-	  if(piece.isNull()) {
-	    break;
-	  } else {
-	    btRequestFactory->addTargetPiece(piece);
-	    numMissingBlock += piece->countMissingBlock();
-	    excludedIndexes.push_back(piece->getIndex());
-	  }
-	}
+        std::vector<size_t> excludedIndexes;
+        excludedIndexes.reserve(btRequestFactory->countTargetPiece());
+        btRequestFactory->getTargetPieceIndexes(excludedIndexes);
+        while(numMissingBlock < maxMissingBlock) {
+          SharedHandle<Piece> piece =
+            _pieceStorage->getMissingFastPiece(peer, excludedIndexes);
+          if(piece.isNull()) {
+            break;
+          } else {
+            btRequestFactory->addTargetPiece(piece);
+            numMissingBlock += piece->countMissingBlock();
+            excludedIndexes.push_back(piece->getIndex());
+          }
+        }
       }
     } else {
-      std::deque<size_t> excludedIndexes;
+      std::vector<size_t> excludedIndexes;
+      excludedIndexes.reserve(btRequestFactory->countTargetPiece());
       btRequestFactory->getTargetPieceIndexes(excludedIndexes);
       while(numMissingBlock < maxMissingBlock) {
-	SharedHandle<Piece> piece =
-	  _pieceStorage->getMissingPiece(peer, excludedIndexes);
-	if(piece.isNull()) {
-	  break;
-	} else {
-	  btRequestFactory->addTargetPiece(piece);
-	  numMissingBlock += piece->countMissingBlock();
-	  excludedIndexes.push_back(piece->getIndex());
-	}
+        SharedHandle<Piece> piece =
+          _pieceStorage->getMissingPiece(peer, excludedIndexes);
+        if(piece.isNull()) {
+          break;
+        } else {
+          btRequestFactory->addTargetPiece(piece);
+          numMissingBlock += piece->countMissingBlock();
+          excludedIndexes.push_back(piece->getIndex());
+        }
       }
     }
   }
 }
 
 void DefaultBtInteractive::addRequests() {
-  size_t MAX_PENDING_REQUEST;
-  if(peer->getLatency() < 500) {
-    MAX_PENDING_REQUEST = 24;
-  } else if(peer->getLatency() < 1500) {
-    MAX_PENDING_REQUEST = 12;
-  } else {
-    MAX_PENDING_REQUEST = 6;
-  }
-  fillPiece(MAX_PENDING_REQUEST);
-
+  fillPiece(_maxOutstandingRequest);
   size_t reqNumToCreate =
-    MAX_PENDING_REQUEST <= dispatcher->countOutstandingRequest() ?
-    0 : MAX_PENDING_REQUEST-dispatcher->countOutstandingRequest();
+    _maxOutstandingRequest <= dispatcher->countOutstandingRequest() ?
+    0 : _maxOutstandingRequest-dispatcher->countOutstandingRequest();
   if(reqNumToCreate > 0) {
-    BtMessages requests;
+    std::vector<SharedHandle<BtMessage> > requests;
+    requests.reserve(reqNumToCreate);
     if(_pieceStorage->isEndGame()) {
-      btRequestFactory->createRequestMessagesOnEndGame(requests, reqNumToCreate);
+      btRequestFactory->createRequestMessagesOnEndGame(requests,reqNumToCreate);
     } else {
       btRequestFactory->createRequestMessages(requests, reqNumToCreate);
     }
@@ -364,6 +383,18 @@ void DefaultBtInteractive::addRequests() {
 
 void DefaultBtInteractive::cancelAllPiece() {
   btRequestFactory->removeAllTargetPiece();
+  if(_metadataGetMode && _downloadContext->getTotalLength() > 0) {
+    std::vector<size_t> metadataRequests =
+      _utMetadataRequestTracker->getAllTrackedIndex();
+    for(std::vector<size_t>::const_iterator i = metadataRequests.begin(),
+          eoi = metadataRequests.end(); i != eoi; ++i) {
+      if(logger->debug()) {
+        logger->debug("Cancel metadata: piece=%lu",
+                      static_cast<unsigned long>(*i));
+      }
+      _pieceStorage->cancelPiece(_pieceStorage->getPiece(*i));
+    }
+  }
 }
 
 void DefaultBtInteractive::sendPendingMessage() {
@@ -371,29 +402,31 @@ void DefaultBtInteractive::sendPendingMessage() {
 }
 
 void DefaultBtInteractive::detectMessageFlooding() {
-  if(floodingCheckPoint.elapsed(FLOODING_CHECK_INTERVAL)) {
+  if(floodingCheckPoint.
+     difference(global::wallclock) >= FLOODING_CHECK_INTERVAL) {
     if(floodingStat.getChokeUnchokeCount() >= 2 ||
        floodingStat.getKeepAliveCount() >= 2) {
       throw DL_ABORT_EX(EX_FLOODING_DETECTED);
     } else {
       floodingStat.reset();
     }
-    floodingCheckPoint.reset();
+    floodingCheckPoint = global::wallclock;
   }
 }
 
 void DefaultBtInteractive::checkActiveInteraction()
 {
+  time_t inactiveTime = inactiveCheckPoint.difference(global::wallclock);
   // To allow aria2 to accept mutially interested peer, disconnect unintersted
   // peer.
   {
     const time_t interval = 30;
     if(!peer->amInterested() && !peer->peerInterested() &&
-       inactiveCheckPoint.elapsed(interval)) {
+       inactiveTime >= interval) {
       // TODO change the message
       throw DL_ABORT_EX
-	(StringFormat("Disconnect peer because we are not interested each other"
-		      " after %u second(s).", interval).str());
+        (StringFormat("Disconnect peer because we are not interested each other"
+                      " after %u second(s).", interval).str());
     }
   }
   // Since the peers which are *just* connected and do nothing to improve
@@ -401,66 +434,102 @@ void DefaultBtInteractive::checkActiveInteraction()
   // are disconnected in a certain time period.
   {
     const time_t interval = 60;
-    if(inactiveCheckPoint.elapsed(interval)) {
+    if(inactiveTime >= interval) {
       throw DL_ABORT_EX
-	(StringFormat(EX_DROP_INACTIVE_CONNECTION, interval).str());
+        (StringFormat(EX_DROP_INACTIVE_CONNECTION, interval).str());
     }
   }
 }
 
 void DefaultBtInteractive::addPeerExchangeMessage()
 {
-  if(_pexCheckPoint.elapsed(UTPexExtensionMessage::DEFAULT_INTERVAL)) {
+  if(_pexCheckPoint.
+     difference(global::wallclock) >= UTPexExtensionMessage::DEFAULT_INTERVAL) {
     UTPexExtensionMessageHandle m
       (new UTPexExtensionMessage(peer->getExtensionMessageID("ut_pex")));
-    const Peers& peers = _peerStorage->getPeers();
+    const std::deque<SharedHandle<Peer> >& peers = _peerStorage->getPeers();
     {
       for(std::deque<SharedHandle<Peer> >::const_iterator i =
-	    peers.begin(); i != peers.end() && !m->freshPeersAreFull(); ++i) {
-	if(peer->ipaddr != (*i)->ipaddr) {
-	  m->addFreshPeer(*i);
-	}
+            peers.begin(), eoi = peers.end();
+          i != eoi && !m->freshPeersAreFull(); ++i) {
+        if(peer->ipaddr != (*i)->ipaddr) {
+          m->addFreshPeer(*i);
+        }
       }
     }
     {
       for(std::deque<SharedHandle<Peer> >::const_reverse_iterator i =
-	    peers.rbegin(); i != peers.rend() && !m->droppedPeersAreFull();
-	  ++i) {
-	if(peer->ipaddr != (*i)->ipaddr) {
-	  m->addDroppedPeer(*i);
-	}
+            peers.rbegin(), eoi = peers.rend();
+          i != eoi && !m->droppedPeersAreFull();
+          ++i) {
+        if(peer->ipaddr != (*i)->ipaddr) {
+          m->addDroppedPeer(*i);
+        }
       }
     }
     BtMessageHandle msg = messageFactory->createBtExtendedMessage(m);
     dispatcher->addMessageToQueue(msg);
-    _pexCheckPoint.reset();
+    _pexCheckPoint = global::wallclock;
   }
 }
 
 void DefaultBtInteractive::doInteractionProcessing() {
-  checkActiveInteraction();
+  if(_metadataGetMode) {
+    sendKeepAlive();
+    _numReceivedMessage = receiveMessages();
+    // PieceStorage is re-initialized with metadata_size in
+    // HandshakeExtensionMessage::doReceivedAction().
+    _pieceStorage =
+      _downloadContext->getOwnerRequestGroup()->getPieceStorage();
 
-  decideChoking();
+    if(peer->getExtensionMessageID("ut_metadata") &&
+       _downloadContext->getTotalLength() > 0) {
+      size_t num = _utMetadataRequestTracker->avail();
+      if(num > 0) {
+        std::vector<SharedHandle<BtMessage> > requests;
+        _utMetadataRequestFactory->create(requests, num, _pieceStorage);
+        dispatcher->addMessageToQueue(requests);
+      }
+      if(_perSecCheckPoint.difference(global::wallclock) >= 1) {
+        _perSecCheckPoint = global::wallclock;
+        // Drop timeout request after queuing message to give a chance
+        // to other connection to request piece.
+        std::vector<size_t> indexes =
+          _utMetadataRequestTracker->removeTimeoutEntry();
+        for(std::vector<size_t>::const_iterator i = indexes.begin(),
+              eoi = indexes.end(); i != eoi; ++i) {
+          _pieceStorage->cancelPiece(_pieceStorage->getPiece(*i));
+        }
+      }
+      if(_pieceStorage->downloadFinished()) {
+        _downloadContext->getOwnerRequestGroup()->setForceHaltRequested
+          (true, RequestGroup::NONE);
+      }
+    }
+  } else {
+    checkActiveInteraction();
 
-  detectMessageFlooding();
+    decideChoking();
 
-  if(_perSecCheckPoint.elapsed(1)) {
-    _perSecCheckPoint.reset();
-    dispatcher->checkRequestSlotAndDoNecessaryThing();
-  }
-  checkHave();
+    detectMessageFlooding();
 
-  sendKeepAlive();
+    if(_perSecCheckPoint.difference(global::wallclock) >= 1) {
+      _perSecCheckPoint = global::wallclock;
+      dispatcher->checkRequestSlotAndDoNecessaryThing();
+    }
+    checkHave();
 
-  _numReceivedMessage = receiveMessages();
+    sendKeepAlive();
+
+    _numReceivedMessage = receiveMessages();
   
-  btRequestFactory->removeCompletedPiece();
+    btRequestFactory->removeCompletedPiece();
 
-  decideInterest();
-  if(!_pieceStorage->downloadFinished()) {
-    addRequests();
+    decideInterest();
+    if(!_pieceStorage->downloadFinished()) {
+      addRequests();
+    }
   }
-
   if(peer->getExtensionMessageID("ut_pex") && _utPexEnabled) {
     addPeerExchangeMessage();
   }
@@ -490,7 +559,11 @@ size_t DefaultBtInteractive::countReceivedMessageInIteration() const
 
 size_t DefaultBtInteractive::countOutstandingRequest()
 {
-  return dispatcher->countOutstandingRequest();
+  if(_metadataGetMode) {
+    return _utMetadataRequestTracker->count();
+  } else {
+    return dispatcher->countOutstandingRequest();
+  }
 }
 
 void DefaultBtInteractive::setBtRuntime
