@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -78,7 +78,7 @@ namespace aria2 {
 PeerInteractionCommand::PeerInteractionCommand
 (int32_t cuid,
  RequestGroup* requestGroup,
- const PeerHandle& p,
+ const SharedHandle<Peer>& p,
  DownloadEngine* e,
  const SharedHandle<BtRuntime>& btRuntime,
  const SharedHandle<PieceStorage>& pieceStorage,
@@ -189,7 +189,8 @@ PeerInteractionCommand::PeerInteractionCommand
     (getOption()->getAsInt(PREF_BT_KEEP_ALIVE_INTERVAL));
   btInteractive->setRequestGroupMan(e->_requestGroupMan);
   btInteractive->setBtMessageFactory(factory);
-  if(metadataGetMode || torrentAttrs[bittorrent::PRIVATE].i() == 0) {
+  if((metadataGetMode || torrentAttrs[bittorrent::PRIVATE].i() == 0) &&
+     !peer->isLocalPeer()) {
     if(getOption()->getAsBool(PREF_ENABLE_PEER_EXCHANGE)) {
       btInteractive->setUTPexEnabled(true);
     }
@@ -237,7 +238,7 @@ PeerInteractionCommand::PeerInteractionCommand
 PeerInteractionCommand::~PeerInteractionCommand() {
   if(peer->getCompletedLength() > 0) {
     _pieceStorage->subtractPieceStats(peer->getBitfield(),
-				      peer->getBitfieldLength());
+                                      peer->getBitfieldLength());
   }
   peer->releaseSessionResource();
 
@@ -265,7 +266,7 @@ bool PeerInteractionCommand::executeInternal() {
     if(btInteractive->countPendingMessage() > 0) {
       btInteractive->sendPendingMessage();
       if(btInteractive->countPendingMessage() > 0) {
-	break;
+        break;
       }
     }
     BtMessageHandle handshakeMessage = btInteractive->receiveHandshake();
@@ -299,15 +300,15 @@ bool PeerInteractionCommand::executeInternal() {
 
       // Writable check to avoid slow seeding
       if(btInteractive->isSendingMessageInProgress()) {
-       	setWriteCheckSocket(socket);
+        setWriteCheckSocket(socket);
       }
 
       if(e->_requestGroupMan->doesOverallDownloadSpeedExceed() ||
-	 _requestGroup->doesDownloadSpeedExceed()) {
-	disableReadCheckSocket();
-	setNoCheck(true);
+         _requestGroup->doesDownloadSpeedExceed()) {
+        disableReadCheckSocket();
+        setNoCheck(true);
       } else {
-	setReadCheckSocket(socket);
+        setReadCheckSocket(socket);
       }
     } else {
       disableReadCheckSocket();
@@ -324,7 +325,7 @@ bool PeerInteractionCommand::executeInternal() {
 // TODO this method removed when PeerBalancerCommand is implemented
 bool PeerInteractionCommand::prepareForNextPeer(time_t wait) {
   if(_peerStorage->isPeerAvailable() && _btRuntime->lessThanEqMinPeers()) {
-    PeerHandle peer = _peerStorage->getUnusedPeer();
+    SharedHandle<Peer> peer = _peerStorage->getUnusedPeer();
     peer->usedBy(e->newCUID());
     PeerInitiateConnectionCommand* command =
       new PeerInitiateConnectionCommand
