@@ -61,7 +61,7 @@ namespace aria2 {
 
 PeerReceiveHandshakeCommand::PeerReceiveHandshakeCommand
 (int32_t cuid,
- const PeerHandle& peer,
+ const SharedHandle<Peer>& peer,
  DownloadEngine* e,
  const SocketHandle& s,
  const SharedHandle<PeerConnection>& peerConnection)
@@ -142,7 +142,9 @@ bool PeerReceiveHandshakeCommand::executeInternal()
            PeerInteractionCommand::RECEIVER_WAIT_HANDSHAKE,
            _peerConnection);
         e->commands.push_back(command);
-        logger->debug(MSG_INCOMING_PEER_CONNECTION, cuid, peer->usedBy());
+        if(logger->debug()) {
+          logger->debug(MSG_INCOMING_PEER_CONNECTION, cuid, peer->usedBy());
+        }
       }
     }
     return true;
