@@ -70,10 +70,11 @@
 
 namespace aria2 {
 
-RequestGroupMan::RequestGroupMan(const RequestGroups& requestGroups,
-                                 unsigned int maxSimultaneousDownloads,
-                                 const Option* option):
-  _reservedGroups(requestGroups),
+RequestGroupMan::RequestGroupMan
+(const std::vector<SharedHandle<RequestGroup> >& requestGroups,
+ unsigned int maxSimultaneousDownloads,
+ const Option* option):
+  _reservedGroups(requestGroups.begin(), requestGroups.end()),
   _logger(LogFactory::getInstance()),
   _maxSimultaneousDownloads(maxSimultaneousDownloads),
   _gidCounter(0),
@@ -82,7 +83,8 @@ RequestGroupMan::RequestGroupMan(const RequestGroups& requestGroups,
   _maxOverallDownloadSpeedLimit
   (option->getAsInt(PREF_MAX_OVERALL_DOWNLOAD_LIMIT)),
   _maxOverallUploadSpeedLimit(option->getAsInt(PREF_MAX_OVERALL_UPLOAD_LIMIT)),
-  _xmlRpc(option->getAsBool(PREF_ENABLE_XML_RPC))
+  _xmlRpc(option->getAsBool(PREF_ENABLE_XML_RPC)),
+  _queueCheck(true)
 {}
 
 bool RequestGroupMan::downloadFinished()
@@ -95,24 +97,36 @@ bool RequestGroupMan::downloadFinished()
   return _requestGroups.empty() && _reservedGroups.empty();
 }
 
-void RequestGroupMan::addRequestGroup(const RequestGroupHandle& group)
+void RequestGroupMan::addRequestGroup
+(const SharedHandle<RequestGroup>& group)
 {
   _requestGroups.push_back(group);
 }
 
-void RequestGroupMan::addReservedGroup(const RequestGroups& groups)
+void RequestGroupMan::addReservedGroup
+(const std::vector<SharedHandle<RequestGroup> >& groups)
 {
+  if(_reservedGroups.empty()) {
+    requestQueueCheck();
+  }
   _reservedGroups.insert(_reservedGroups.end(), groups.begin(), groups.end());
 }
 
-void RequestGroupMan::addReservedGroup(const SharedHandle<RequestGroup>& group)
+void RequestGroupMan::addReservedGroup
+(const SharedHandle<RequestGroup>& group)
 {
+  if(_reservedGroups.empty()) {
+    requestQueueCheck();
+  }
   _reservedGroups.push_back(group);
 }
 
 void RequestGroupMan::insertReservedGroup
-(size_t pos, const std::deque<SharedHandle<RequestGroup> >& groups)
+(size_t pos, const std::vector<SharedHandle<RequestGroup> >& groups)
 {
+  if(_reservedGroups.empty()) {
+    requestQueueCheck();
+  }
   _reservedGroups.insert
     (_reservedGroups.begin()+std::min(_reservedGroups.size(), pos),
      groups.begin(), groups.end());
@@ -121,6 +135,9 @@ void RequestGroupMan::insertReservedGroup
 void RequestGroupMan::insertReservedGroup
 (size_t pos, const SharedHandle<RequestGroup>& group)
 {
+  if(_reservedGroups.empty()) {
+    requestQueueCheck();
+  }
   _reservedGroups.insert
     (_reservedGroups.begin()+std::min(_reservedGroups.size(), pos), group);
 }
@@ -130,7 +147,7 @@ size_t RequestGroupMan::countRequestGroup() const
   return _requestGroups.size();
 }
 
-RequestGroupHandle RequestGroupMan::getRequestGroup(size_t index) const
+SharedHandle<RequestGroup> RequestGroupMan::getRequestGroup(size_t index) const
 {
   if(index < _requestGroups.size()) {
     return _requestGroups[index];
@@ -299,7 +316,6 @@ static void executeStopHook
 class ProcessStoppedRequestGroup {
 private:
   DownloadEngine* _e;
-  std::deque<SharedHandle<RequestGroup> >& _reservedGroups;
   std::deque<SharedHandle<DownloadResult> >& _downloadResults;
   Logger* _logger;
 
@@ -321,16 +337,24 @@ private:
 public:
   ProcessStoppedRequestGroup
   (DownloadEngine* e,
-   std::deque<SharedHandle<RequestGroup> >& reservedGroups,
    std::deque<SharedHandle<DownloadResult> >& downloadResults):
     _e(e),
-    _reservedGroups(reservedGroups),
     _downloadResults(downloadResults),
     _logger(LogFactory::getInstance()) {}
 
   void operator()(const SharedHandle<RequestGroup>& group)
   {
     if(group->getNumCommand() == 0) {
+      SharedHandle<DownloadContext> dctx = group->getDownloadContext();
+      // DownloadContext::resetDownloadStopTime() is only called when
+      // download completed. If
+      // DownloadContext::getDownloadStopTime().isZero() is true, then
+      // there is a possibility that the download is error or
+      // in-progress and resetDownloadStopTime() is not called. So
+      // call it here.
+      if(dctx->getDownloadStopTime().isZero()) {
+        dctx->resetDownloadStopTime();
+      }
       try {
         group->closeFile();
         if(group->downloadFinished()) {
@@ -342,14 +366,15 @@ public:
           } else {
             group->saveControlFile();
           }
-          RequestGroups nextGroups;
+          std::vector<SharedHandle<RequestGroup> > nextGroups;
           group->postDownloadProcessing(nextGroups);
           if(!nextGroups.empty()) {
-            _logger->debug
-              ("Adding %lu RequestGroups as a result of PostDownloadHandler.",
-               static_cast<unsigned long>(nextGroups.size()));
-            _reservedGroups.insert(_reservedGroups.begin(),
-                                   nextGroups.begin(), nextGroups.end());
+            if(_logger->debug()) {
+              _logger->debug
+                ("Adding %lu RequestGroups as a result of PostDownloadHandler.",
+                 static_cast<unsigned long>(nextGroups.size()));
+            }
+            _e->_requestGroupMan->insertReservedGroup(0, nextGroups);
           }
         } else {
           group->saveControlFile();
@@ -380,10 +405,10 @@ public:
       if(!group->getSegmentMan().isNull()) {
         bool singleConnection =
           group->getSegmentMan()->getPeerStats().size() == 1;
-        const std::deque<SharedHandle<PeerStat> >& peerStats =
+        const std::vector<SharedHandle<PeerStat> >& peerStats =
           group->getSegmentMan()->getFastestPeerStats();
-        for(std::deque<SharedHandle<PeerStat> >::const_iterator i =
-              peerStats.begin(); i != peerStats.end(); ++i) {
+        for(std::vector<SharedHandle<PeerStat> >::const_iterator i =
+              peerStats.begin(), eoi = peerStats.end(); i != eoi; ++i) {
           if((*i)->getHostname().empty() || (*i)->getProtocol().empty()) {
             continue;
           }
@@ -427,8 +452,7 @@ void RequestGroupMan::removeStoppedGroup(DownloadEngine* e)
   updateServerStat();
 
   std::for_each(_requestGroups.begin(), _requestGroups.end(),
-                ProcessStoppedRequestGroup(e, _reservedGroups,
-                                           _downloadResults));
+                ProcessStoppedRequestGroup(e, _downloadResults));
 
   std::deque<SharedHandle<RequestGroup> >::iterator i =
     std::remove_if(_requestGroups.begin(),
@@ -440,8 +464,10 @@ void RequestGroupMan::removeStoppedGroup(DownloadEngine* e)
 
   size_t numRemoved = numPrev-_requestGroups.size();
   if(numRemoved > 0) {
-    _logger->debug("%lu RequestGroup(s) deleted.",
-                   static_cast<unsigned long>(numRemoved));
+    if(_logger->debug()) {
+      _logger->debug("%lu RequestGroup(s) deleted.",
+                     static_cast<unsigned long>(numRemoved));
+    }
   }
 }
 
@@ -463,7 +489,7 @@ void RequestGroupMan::configureRequestGroup
 }
 
 static void createInitialCommand(const SharedHandle<RequestGroup>& requestGroup,
-                                 std::deque<Command*>& commands,
+                                 std::vector<Command*>& commands,
                                  DownloadEngine* e)
 {
   requestGroup->createInitialCommand(commands, e);
@@ -475,13 +501,13 @@ void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
   if(_maxSimultaneousDownloads <= _requestGroups.size()) {
     return;
   }
-  RequestGroups temp;
+  std::vector<SharedHandle<RequestGroup> > temp;
   unsigned int count = 0;
   size_t num = _maxSimultaneousDownloads-_requestGroups.size();
   while(count < num && !_reservedGroups.empty()) {
-    RequestGroupHandle groupToAdd = _reservedGroups.front();
+    SharedHandle<RequestGroup> groupToAdd = _reservedGroups.front();
     _reservedGroups.pop_front();
-    Commands commands;
+    std::vector<Command*> commands;
     try {
       if(!groupToAdd->isDependencyResolved()) {
         temp.push_back(groupToAdd);
@@ -489,6 +515,10 @@ void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
       }
       configureRequestGroup(groupToAdd);
       createInitialCommand(groupToAdd, commands, e);
+      groupToAdd->setRequestGroupMan(this);
+      if(commands.empty()) {
+        requestQueueCheck();
+      }
       _requestGroups.push_back(groupToAdd);
       ++count;
       e->addCommand(commands);
@@ -496,10 +526,14 @@ void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
       executeStartHook(groupToAdd, e->option);
     } catch(RecoverableException& ex) {
       _logger->error(EX_EXCEPTION_CAUGHT, ex);
-      _logger->debug("Deleting temporal commands.");
+      if(_logger->debug()) {
+        _logger->debug("Deleting temporal commands.");
+      }
       std::for_each(commands.begin(), commands.end(), Deleter());
       commands.clear();
-      _logger->debug("Commands deleted");
+      if(_logger->debug()) {
+        _logger->debug("Commands deleted");
+      }
       groupToAdd->releaseRuntimeResource(e);
       _downloadResults.push_back(groupToAdd->createDownloadResult());
     }
@@ -509,14 +543,16 @@ void RequestGroupMan::fillRequestGroupFromReserver(DownloadEngine* e)
   }
   if(count > 0) {
     e->setNoWait(true);
-    _logger->debug("%d RequestGroup(s) added.", count);
+    if(_logger->debug()) {
+      _logger->debug("%d RequestGroup(s) added.", count);
+    }
   }
 }
 
 void RequestGroupMan::save()
 {
-  for(RequestGroups::iterator itr = _requestGroups.begin();
-      itr != _requestGroups.end(); ++itr) {
+  for(std::deque<SharedHandle<RequestGroup> >::const_iterator itr =
+        _requestGroups.begin(), eoi = _requestGroups.end(); itr != eoi; ++itr) {
     if((*itr)->allDownloadFinished()) {
       (*itr)->removeControlFile();
     } else {
@@ -531,8 +567,8 @@ void RequestGroupMan::save()
 
 void RequestGroupMan::closeFile()
 {
-  for(RequestGroups::iterator itr = _requestGroups.begin();
-      itr != _requestGroups.end(); ++itr) {
+  for(std::deque<SharedHandle<RequestGroup> >::const_iterator itr =
+        _requestGroups.begin(), eoi = _requestGroups.end(); itr != eoi; ++itr) {
     (*itr)->closeFile();
   }
 }
@@ -543,8 +579,9 @@ RequestGroupMan::DownloadStat RequestGroupMan::getDownloadStat() const
   size_t error = 0;
   size_t inprogress = 0;
   downloadresultcode::RESULT lastError = downloadresultcode::FINISHED;
-  for(std::deque<SharedHandle<DownloadResult> >::const_iterator itr = _downloadResults.begin();
-      itr != _downloadResults.end(); ++itr) {
+  for(std::deque<SharedHandle<DownloadResult> >::const_iterator itr =
+        _downloadResults.begin(), eoi = _downloadResults.end();
+      itr != eoi; ++itr) {
     if((*itr)->result == downloadresultcode::FINISHED) {
       ++finished;
     } else {
@@ -552,8 +589,8 @@ RequestGroupMan::DownloadStat RequestGroupMan::getDownloadStat() const
       lastError = (*itr)->result;
     }
   }
-  for(RequestGroups::const_iterator itr = _requestGroups.begin();
-      itr != _requestGroups.end(); ++itr) {
+  for(std::deque<SharedHandle<RequestGroup> >::const_iterator itr =
+        _requestGroups.begin(), eoi = _requestGroups.end(); itr != eoi; ++itr) {
     DownloadResultHandle result = (*itr)->createDownloadResult();
     if(result->result == downloadresultcode::FINISHED) {
       ++finished;
@@ -588,8 +625,9 @@ void RequestGroupMan::showDownloadResults(std::ostream& o) const
   int err = 0;
   int inpr = 0;
 
-  for(std::deque<SharedHandle<DownloadResult> >::const_iterator itr = _downloadResults.begin();
-      itr != _downloadResults.end(); ++itr) {
+  for(std::deque<SharedHandle<DownloadResult> >::const_iterator itr =
+        _downloadResults.begin(), eoi = _downloadResults.end();
+      itr != eoi; ++itr) {
     std::string status;
     if((*itr)->result == downloadresultcode::FINISHED) {
       status = MARK_OK;
@@ -603,8 +641,8 @@ void RequestGroupMan::showDownloadResults(std::ostream& o) const
     }
     o << formatDownloadResult(status, *itr) << "\n";
   }
-  for(RequestGroups::const_iterator itr = _requestGroups.begin();
-      itr != _requestGroups.end(); ++itr) {
+  for(std::deque<SharedHandle<RequestGroup> >::const_iterator itr =
+        _requestGroups.begin(), eoi = _requestGroups.end(); itr != eoi; ++itr) {
     DownloadResultHandle result = (*itr)->createDownloadResult();
     std::string status;
     if(result->result == downloadresultcode::FINISHED) {
@@ -681,9 +719,9 @@ bool RequestGroupMan::isSameFileBeingDownloaded(RequestGroup* requestGroup) cons
   if(!requestGroup->isPreLocalFileCheckEnabled()) {
     return false;
   }
-  std::deque<std::string> files;
-  for(RequestGroups::const_iterator itr = _requestGroups.begin();
-      itr != _requestGroups.end(); ++itr) {
+  std::vector<std::string> files;
+  for(std::deque<SharedHandle<RequestGroup> >::const_iterator itr =
+        _requestGroups.begin(), eoi = _requestGroups.end(); itr != eoi; ++itr) {
     if((*itr).get() != requestGroup) {
       const std::vector<SharedHandle<FileEntry> >& entries =
         (*itr)->getDownloadContext()->getFileEntries();
@@ -701,17 +739,17 @@ bool RequestGroupMan::isSameFileBeingDownloaded(RequestGroup* requestGroup) cons
 
 void RequestGroupMan::halt()
 {
-  for(RequestGroups::const_iterator itr = _requestGroups.begin();
-      itr != _requestGroups.end(); ++itr) {
-    (*itr)->setHaltRequested(true);
+  for(std::deque<SharedHandle<RequestGroup> >::const_iterator i =
+        _requestGroups.begin(), eoi = _requestGroups.end(); i != eoi; ++i) {
+    (*i)->setHaltRequested(true);
   }
 }
 
 void RequestGroupMan::forceHalt()
 {
-  for(RequestGroups::const_iterator itr = _requestGroups.begin();
-      itr != _requestGroups.end(); ++itr) {
-    (*itr)->setForceHaltRequested(true);
+  for(std::deque<SharedHandle<RequestGroup> >::const_iterator i =
+        _requestGroups.begin(), eoi = _requestGroups.end(); i != eoi; ++i) {
+    (*i)->setForceHaltRequested(true);
   }
 }
 
@@ -719,7 +757,7 @@ TransferStat RequestGroupMan::calculateStat()
 {
   TransferStat s;
   for(std::deque<SharedHandle<RequestGroup> >::const_iterator i =
-        _requestGroups.begin(); i != _requestGroups.end(); ++i) {
+        _requestGroups.begin(), eoi = _requestGroups.end(); i != eoi; ++i) {
     s += (*i)->calculateStat();
   }
   return s;
@@ -729,7 +767,7 @@ SharedHandle<DownloadResult>
 RequestGroupMan::findDownloadResult(int32_t gid) const
 {
   for(std::deque<SharedHandle<DownloadResult> >::const_iterator i =
-        _downloadResults.begin(); i != _downloadResults.end(); ++i) {
+        _downloadResults.begin(), eoi = _downloadResults.end(); i != eoi; ++i) {
     if((*i)->gid == gid) {
       return *i;
     }
