@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -51,17 +51,18 @@
 #include "DownloadContext.h"
 #include "Piece.h"
 #include "FileEntry.h"
+#include "wallclock.h"
 
 namespace aria2 {
 
-SegmentEntry::SegmentEntry(cuid_t cuid, const SegmentHandle& segment):
+SegmentEntry::SegmentEntry(cuid_t cuid, const SharedHandle<Segment>& segment):
   cuid(cuid), segment(segment) {}
 
 SegmentEntry::~SegmentEntry() {}
 
 SegmentMan::SegmentMan(const Option* option,
-		       const SharedHandle<DownloadContext>& downloadContext,
-		       const PieceStorageHandle& pieceStorage):
+                       const SharedHandle<DownloadContext>& downloadContext,
+                       const PieceStorageHandle& pieceStorage):
   _option(option),
   logger(LogFactory::getInstance()),
   _downloadContext(downloadContext),
@@ -69,7 +70,7 @@ SegmentMan::SegmentMan(const Option* option,
   _lastPeerStatDlspdMapUpdated(0),
   _cachedDlspd(0),
   _ignoreBitfield(downloadContext->getPieceLength(),
-		  downloadContext->getTotalLength())
+                  downloadContext->getTotalLength())
 {
   _ignoreBitfield.enableFilter();
 }
@@ -95,7 +96,7 @@ uint64_t SegmentMan::getTotalLength() const
   if(_pieceStorage.isNull()) {
     return 0;
   } else {
-      return _pieceStorage->getTotalLength();
+    return _pieceStorage->getTotalLength();
   }
 }
 
@@ -110,15 +111,16 @@ void SegmentMan::setDownloadContext
   _downloadContext = downloadContext;
 }
 
-SegmentHandle SegmentMan::checkoutSegment(cuid_t cuid,
-					  const PieceHandle& piece)
+SharedHandle<Segment> SegmentMan::checkoutSegment
+(cuid_t cuid, const SharedHandle<Piece>& piece)
 {
   if(piece.isNull()) {
     return SharedHandle<Segment>();
   }
-  logger->debug("Attach segment#%d to CUID#%d.", piece->getIndex(), cuid);
-
-  SegmentHandle segment;
+  if(logger->debug()) {
+    logger->debug("Attach segment#%d to CUID#%d.", piece->getIndex(), cuid);
+  }
+  SharedHandle<Segment> segment;
   if(piece->getLength() == 0) {
     segment.reset(new GrowSegment(piece));
   } else {
@@ -126,36 +128,39 @@ SegmentHandle SegmentMan::checkoutSegment(cuid_t cuid,
   }
   SegmentEntryHandle entry(new SegmentEntry(cuid, segment));
   usedSegmentEntries.push_back(entry);
-
-  logger->debug("index=%d, length=%d, segmentLength=%d, writtenLength=%d",
-		segment->getIndex(),
-		segment->getLength(),
-		segment->getSegmentLength(),
-		segment->getWrittenLength());
+  if(logger->debug()) {
+    logger->debug("index=%d, length=%d, segmentLength=%d, writtenLength=%d",
+                  segment->getIndex(),
+                  segment->getLength(),
+                  segment->getSegmentLength(),
+                  segment->getWrittenLength());
+  }
   if(piece->getLength() > 0) {
     std::map<size_t, size_t>::iterator positr =
       _segmentWrittenLengthMemo.find(segment->getIndex());
     if(positr != _segmentWrittenLengthMemo.end()) {
       const size_t writtenLength = (*positr).second;
-      logger->debug("writtenLength(in memo)=%d, writtenLength=%d",
-		    writtenLength, segment->getWrittenLength());
+      if(logger->debug()) {
+        logger->debug("writtenLength(in memo)=%d, writtenLength=%d",
+                      writtenLength, segment->getWrittenLength());
+      }
       //  If the difference between cached writtenLength and segment's
       //  writtenLength is less than one block, we assume that these
       //  missing bytes are already downloaded.
       if(segment->getWrittenLength() < writtenLength &&
-	 writtenLength-segment->getWrittenLength() < piece->getBlockLength()) {
-	segment->updateWrittenLength(writtenLength-segment->getWrittenLength());
+         writtenLength-segment->getWrittenLength() < piece->getBlockLength()) {
+        segment->updateWrittenLength(writtenLength-segment->getWrittenLength());
       }
     }
   }
   return segment;
 }
 
-void SegmentMan::getInFlightSegment(std::deque<SharedHandle<Segment> >& segments,
-				    cuid_t cuid)
+void SegmentMan::getInFlightSegment
+(std::vector<SharedHandle<Segment> >& segments, cuid_t cuid)
 {
-  for(SegmentEntries::iterator itr = usedSegmentEntries.begin();
-      itr != usedSegmentEntries.end(); ++itr) {
+  for(SegmentEntries::const_iterator itr = usedSegmentEntries.begin(),
+        eoi = usedSegmentEntries.end(); itr != eoi; ++itr) {
     const SegmentEntryHandle& segmentEntry = *itr;
     if(segmentEntry->cuid == cuid) {
       segments.push_back(segmentEntry->segment);
@@ -163,27 +168,28 @@ void SegmentMan::getInFlightSegment(std::deque<SharedHandle<Segment> >& segments
   }
 }
 
-SegmentHandle SegmentMan::getSegment(cuid_t cuid) {
-  PieceHandle piece =
+SharedHandle<Segment> SegmentMan::getSegment(cuid_t cuid) {
+  SharedHandle<Piece> piece =
     _pieceStorage->getSparseMissingUnusedPiece
     (_ignoreBitfield.getFilterBitfield(),_ignoreBitfield.getBitfieldLength());
   return checkoutSegment(cuid, piece);
 }
 
-void SegmentMan::getSegment(std::deque<SharedHandle<Segment> >& segments,
-			    cuid_t cuid,
-			    const SharedHandle<FileEntry>& fileEntry,
-			    size_t maxSegments)
+void SegmentMan::getSegment
+(std::vector<SharedHandle<Segment> >& segments,
+ cuid_t cuid,
+ const SharedHandle<FileEntry>& fileEntry,
+ size_t maxSegments)
 {
   BitfieldMan filter(_ignoreBitfield);
   filter.enableFilter();
   filter.addNotFilter(fileEntry->getOffset(), fileEntry->getLength());
-  std::deque<SharedHandle<Segment> > pending;
+  std::vector<SharedHandle<Segment> > pending;
   while(segments.size() < maxSegments) {
     SharedHandle<Segment> segment =
       checkoutSegment(cuid,
-		      _pieceStorage->getSparseMissingUnusedPiece
-		      (filter.getFilterBitfield(), filter.getBitfieldLength()));
+                      _pieceStorage->getSparseMissingUnusedPiece
+                      (filter.getFilterBitfield(), filter.getBitfieldLength()));
     if(segment.isNull()) {
       break;
     }
@@ -194,13 +200,13 @@ void SegmentMan::getSegment(std::deque<SharedHandle<Segment> >& segments,
       segments.push_back(segment);
     }
   }
-  for(std::deque<SharedHandle<Segment> >::const_iterator i = pending.begin();
-      i != pending.end(); ++i) {
+  for(std::vector<SharedHandle<Segment> >::const_iterator i = pending.begin(),
+        eoi = pending.end(); i != eoi; ++i) {
     cancelSegment(cuid, *i);
   }
 }
 
-SegmentHandle SegmentMan::getSegment(cuid_t cuid, size_t index) {
+SharedHandle<Segment> SegmentMan::getSegment(cuid_t cuid, size_t index) {
   if(_downloadContext->getNumPieces() <= index) {
     return SharedHandle<Segment>();
   }
@@ -211,16 +217,19 @@ void SegmentMan::cancelSegment(const SharedHandle<Segment>& segment)
 {
   _pieceStorage->cancelPiece(segment->getPiece());
   _segmentWrittenLengthMemo[segment->getIndex()] = segment->getWrittenLength();
-  logger->debug("Memorized segment index=%u, writtenLength=%u",
-		segment->getIndex(), segment->getWrittenLength());
+  if(logger->debug()) {
+    logger->debug("Memorized segment index=%u, writtenLength=%u",
+                  segment->getIndex(), segment->getWrittenLength());
+  }
 }
 
 void SegmentMan::cancelSegment(cuid_t cuid) {
-  for(SegmentEntries::iterator itr = usedSegmentEntries.begin();
-      itr != usedSegmentEntries.end();) {
+  for(SegmentEntries::iterator itr = usedSegmentEntries.begin(),
+        eoi = usedSegmentEntries.end(); itr != eoi;) {
     if((*itr)->cuid == cuid) {
       cancelSegment((*itr)->segment);
       itr = usedSegmentEntries.erase(itr);
+      eoi = usedSegmentEntries.end();
     } else {
       ++itr;
     }
@@ -230,11 +239,12 @@ void SegmentMan::cancelSegment(cuid_t cuid) {
 void SegmentMan::cancelSegment
 (cuid_t cuid, const SharedHandle<Segment>& segment)
 {
-  for(SegmentEntries::iterator itr = usedSegmentEntries.begin();
-      itr != usedSegmentEntries.end();) {
+  for(SegmentEntries::iterator itr = usedSegmentEntries.begin(),
+        eoi = usedSegmentEntries.end(); itr != eoi;) {
     if((*itr)->cuid == cuid && (*itr)->segment == segment) {
       cancelSegment((*itr)->segment);
       itr = usedSegmentEntries.erase(itr);
+      //eoi = usedSegmentEntries.end();
       break;
     } else {
       ++itr;
@@ -244,9 +254,9 @@ void SegmentMan::cancelSegment
 
 class FindSegmentEntry {
 private:
-  SegmentHandle _segment;
+  SharedHandle<Segment> _segment;
 public:
-  FindSegmentEntry(const SegmentHandle& segment):_segment(segment) {}
+  FindSegmentEntry(const SharedHandle<Segment>& segment):_segment(segment) {}
 
   bool operator()(const SegmentEntryHandle& segmentEntry) const
   {
@@ -254,12 +264,13 @@ public:
   }
 };
 
-bool SegmentMan::completeSegment(cuid_t cuid, const SegmentHandle& segment) {
+bool SegmentMan::completeSegment
+(cuid_t cuid, const SharedHandle<Segment>& segment) {
   _pieceStorage->completePiece(segment->getPiece());
   _pieceStorage->advertisePiece(cuid, segment->getPiece()->getIndex());
   SegmentEntries::iterator itr = std::find_if(usedSegmentEntries.begin(),
-					      usedSegmentEntries.end(),
-					      FindSegmentEntry(segment));
+                                              usedSegmentEntries.end(),
+                                              FindSegmentEntry(segment));
   if(itr == usedSegmentEntries.end()) {
     return false;
   } else {
@@ -282,8 +293,8 @@ uint64_t SegmentMan::getDownloadLength() const {
 
 void SegmentMan::registerPeerStat(const SharedHandle<PeerStat>& peerStat)
 {
-  for(std::deque<SharedHandle<PeerStat> >::iterator i = peerStats.begin();
-      i != peerStats.end(); ++i) {
+  for(std::vector<SharedHandle<PeerStat> >::iterator i = peerStats.begin(),
+        eoi = peerStats.end(); i != eoi; ++i) {
     if((*i)->getStatus() == PeerStat::IDLE) {
       *i = peerStat;
       return;
@@ -308,9 +319,9 @@ public:
 
 void SegmentMan::updateFastestPeerStat(const SharedHandle<PeerStat>& peerStat)
 {
-  std::deque<SharedHandle<PeerStat> >::iterator i =
+  std::vector<SharedHandle<PeerStat> >::iterator i =
     std::find_if(_fastestPeerStats.begin(), _fastestPeerStats.end(),
-		 PeerStatHostProtoEqual(peerStat));
+                 PeerStatHostProtoEqual(peerStat));
   if(i == _fastestPeerStats.end()) {
     _fastestPeerStats.push_back(peerStat);
   } else if((*i)->getAvgDownloadSpeed() < peerStat->getAvgDownloadSpeed()) {
@@ -326,15 +337,15 @@ void SegmentMan::updateFastestPeerStat(const SharedHandle<PeerStat>& peerStat)
 unsigned int SegmentMan::calculateDownloadSpeed()
 {
   unsigned int speed = 0;
-  if(_lastPeerStatDlspdMapUpdated.elapsedInMillis(250)) {
-    _lastPeerStatDlspdMapUpdated.reset();
+  if(_lastPeerStatDlspdMapUpdated.differenceInMillis(global::wallclock) >= 250){
+    _lastPeerStatDlspdMapUpdated = global::wallclock;
     _peerStatDlspdMap.clear();
-    for(std::deque<SharedHandle<PeerStat> >::const_iterator i =
-	  peerStats.begin(); i != peerStats.end(); ++i) {
+    for(std::vector<SharedHandle<PeerStat> >::const_iterator i =
+          peerStats.begin(), eoi = peerStats.end(); i != eoi; ++i) {
       if((*i)->getStatus() == PeerStat::ACTIVE) {
-	unsigned int s = (*i)->calculateDownloadSpeed();
-	_peerStatDlspdMap[(*i)->getCuid()] = s;
-	speed += s;
+        unsigned int s = (*i)->calculateDownloadSpeed();
+        _peerStatDlspdMap[(*i)->getCuid()] = s;
+        speed += s;
       }
     }
     _cachedDlspd = speed;
@@ -368,7 +379,7 @@ public:
 uint64_t SegmentMan::calculateSessionDownloadLength() const
 {
   return std::accumulate(_fastestPeerStats.begin(), _fastestPeerStats.end(),
-			 0LL, PeerStatDownloadLengthOperator());
+                         0LL, PeerStatDownloadLengthOperator());
 }
 
 size_t SegmentMan::countFreePieceFrom(size_t index) const
@@ -384,6 +395,12 @@ size_t SegmentMan::countFreePieceFrom(size_t index) const
 
 void SegmentMan::ignoreSegmentFor(const SharedHandle<FileEntry>& fileEntry)
 {
+  if(logger->debug()) {
+    logger->debug("ignoring segment for path=%s, offset=%s, length=%s",
+                  fileEntry->getPath().c_str(),
+                  util::itos(fileEntry->getOffset()).c_str(),
+                  util::uitos(fileEntry->getLength()).c_str());
+  }
   _ignoreBitfield.addFilter(fileEntry->getOffset(), fileEntry->getLength());
 }
 
