@@ -91,13 +91,15 @@ bool TrackerWatcherCommand::execute() {
     }
   }
   if(_btAnnounce->noMoreAnnounce()) {
-    logger->debug("no more announce");
+    if(logger->debug()) {
+      logger->debug("no more announce");
+    }
     return true;
   }
   if(_trackerRequestGroup.isNull()) {
     _trackerRequestGroup = createAnnounce();
     if(!_trackerRequestGroup.isNull()) {
-      std::deque<Command*> commands;
+      std::vector<Command*> commands;
       try {
         _trackerRequestGroup->createInitialCommand(commands, e);
       } catch(RecoverableException& ex) {
@@ -106,7 +108,9 @@ bool TrackerWatcherCommand::execute() {
         commands.clear();
       }
       e->addCommand(commands);
-      logger->debug("added tracker request command");
+      if(logger->debug()) {
+        logger->debug("added tracker request command");
+      }
     }
   } else if(_trackerRequestGroup->downloadFinished()){
     try {
@@ -136,7 +140,7 @@ bool TrackerWatcherCommand::execute() {
 }
 
 std::string TrackerWatcherCommand::getTrackerResponse
-(const RequestGroupHandle& requestGroup)
+(const SharedHandle<RequestGroup>& requestGroup)
 {
   std::stringstream strm;
   unsigned char data[2048];
@@ -160,7 +164,7 @@ void TrackerWatcherCommand::processTrackerResponse
     (reinterpret_cast<const unsigned char*>(trackerResponse.c_str()),
      trackerResponse.size());
   while(!_btRuntime->isHalt() && _btRuntime->lessThanMinPeers()) {
-    PeerHandle peer = _peerStorage->getUnusedPeer();
+    SharedHandle<Peer> peer = _peerStorage->getUnusedPeer();
     if(peer.isNull()) {
       break;
     }
@@ -171,12 +175,15 @@ void TrackerWatcherCommand::processTrackerResponse
     command->setPeerStorage(_peerStorage);
     command->setPieceStorage(_pieceStorage);
     e->commands.push_back(command);
-    logger->debug("CUID#%d - Adding new command CUID#%d", cuid, peer->usedBy());
+    if(logger->debug()) {
+      logger->debug("CUID#%d - Adding new command CUID#%d",
+                    cuid, peer->usedBy());
+    }
   }
 }
 
-RequestGroupHandle TrackerWatcherCommand::createAnnounce() {
-  RequestGroupHandle rg;
+SharedHandle<RequestGroup> TrackerWatcherCommand::createAnnounce() {
+  SharedHandle<RequestGroup> rg;
   if(_btAnnounce->isAnnounceReady()) {
     rg = createRequestGroup(_btAnnounce->getAnnounceUrl());
     _btAnnounce->announceStart(); // inside it, trackers++.
@@ -202,18 +209,22 @@ static bool backupTrackerIsAvailable
   }
 }
 
-RequestGroupHandle
+SharedHandle<RequestGroup>
 TrackerWatcherCommand::createRequestGroup(const std::string& uri)
 {
-  std::deque<std::string> uris;
+  std::vector<std::string> uris;
   uris.push_back(uri);
-  RequestGroupHandle rg(new RequestGroup(getOption()));
+  SharedHandle<RequestGroup> rg(new RequestGroup(getOption()));
   // If backup tracker is available, only try 2 times for each tracker
   // and if they all fails, then try next one.
   if(backupTrackerIsAvailable(_requestGroup->getDownloadContext())) {
-    logger->debug("This is multi-tracker announce.");
+    if(logger->debug()) {
+      logger->debug("This is multi-tracker announce.");
+    }
   } else {
-    logger->debug("This is single-tracker announce.");
+    if(logger->debug()) {
+      logger->debug("This is single-tracker announce.");
+    }
   }
   rg->getOption()->put(PREF_MAX_TRIES, "2");
   // TODO When dry-run mode becomes available in BitTorrent, set
