@@ -49,18 +49,23 @@ UTMetadataRequestFactory::UTMetadataRequestFactory():
   _logger(LogFactory::getInstance()) {}
 
 void UTMetadataRequestFactory::create
-(std::deque<SharedHandle<BtMessage> >& msgs, size_t num,
+(std::vector<SharedHandle<BtMessage> >& msgs, size_t num,
  const SharedHandle<PieceStorage>& pieceStorage)
 {
-  for(size_t index = 0; index < _dctx->getNumPieces() && num; ++index) {
+  for(size_t index = 0, numPieces = _dctx->getNumPieces();
+      index < numPieces && num; ++index) {
     SharedHandle<Piece> p = pieceStorage->getMissingPiece(index);
     if(p.isNull()) {
-      _logger->debug("ut_metadata piece %lu is used or already acquired.");
+      if(_logger->debug()) {
+        _logger->debug("ut_metadata piece %lu is used or already acquired.");
+      }
       continue;
     }
     --num;
-    _logger->debug("Creating ut_metadata request index=%lu",
-                   static_cast<unsigned long>(index));
+    if(_logger->debug()) {
+      _logger->debug("Creating ut_metadata request index=%lu",
+                     static_cast<unsigned long>(index));
+    }
     SharedHandle<UTMetadataRequestExtensionMessage> m
       (new UTMetadataRequestExtensionMessage
        (_peer->getExtensionMessageID("ut_metadata")));
