@@ -16,7 +16,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
  *
  * In addition, as a special exception, the copyright holders give
  * permission to link the code of portions of this program with the
@@ -75,6 +75,9 @@
 #include "A2STR.h"
 #include "array_fun.h"
 #include "a2functional.h"
+#ifdef ENABLE_MESSAGE_DIGEST
+# include "MessageDigestHelper.h"
+#endif // ENABLE_MESSAGE_DIGEST
 
 // For libc6 which doesn't define ULLONG_MAX properly because of broken limits.h
 #ifndef ULLONG_MAX
@@ -112,7 +115,7 @@ void split(std::pair<std::string, std::string>& hp, const std::string& src, char
   hp.second = A2STR::NIL;
   std::string::size_type p = src.find(delim);
   if(p == std::string::npos) {
-    hp.first = src;
+    hp.first = trim(src);
     hp.second = A2STR::NIL;
   } else {
     hp.first = trim(src.substr(0, p));
@@ -127,7 +130,7 @@ std::pair<std::string, std::string> split(const std::string& src, const std::str
   hp.second = A2STR::NIL;
   std::string::size_type p = src.find_first_of(delims);
   if(p == std::string::npos) {
-    hp.first = src;
+    hp.first = trim(src);
     hp.second = A2STR::NIL;
   } else {
     hp.first = trim(src.substr(0, p));
@@ -136,13 +139,36 @@ std::pair<std::string, std::string> split(const std::string& src, const std::str
   return hp;
 }
 
+std::string itos(int64_t value, bool comma)
+{
+  bool flag = false;
+  std::string str;
+  if(value < 0) {
+    if(value == INT64_MIN) {
+      if(comma) {
+        str = "-9,223,372,036,854,775,808";
+      } else {
+        str = "-9223372036854775808";
+      }
+      return str;
+    }
+    flag = true;
+    value = -value;
+  }
+  str = uitos(value, comma);
+  if(flag) {
+    str.insert(str.begin(), '-');
+  }
+  return str;
+}
+
 int64_t difftv(struct timeval tv1, struct timeval tv2) {
   if((tv1.tv_sec < tv2.tv_sec) ||
      ((tv1.tv_sec == tv2.tv_sec) && (tv1.tv_usec < tv2.tv_usec))) {
     return 0;
   }
   return ((int64_t)(tv1.tv_sec-tv2.tv_sec)*1000000+
-	  tv1.tv_usec-tv2.tv_usec);
+          tv1.tv_usec-tv2.tv_usec);
 }
 
 int32_t difftvsec(struct timeval tv1, struct timeval tv2) {
@@ -198,6 +224,31 @@ std::string replace(const std::string& target, const std::string& oldstr, const
   return result;
 }
 
+bool isAlpha(const char c)
+{
+  return ('A' <= c && c <= 'Z') || ('a' <= c && c <= 'z');
+}
+
+bool isDigit(const char c)
+{
+  return '0' <= c && c <= '9';
+}
+
+bool isHexDigit(const char c)
+{
+  return isDigit(c) || ('A' <= c && c <= 'F') || ('a' <= c && c <= 'f');
+}
+
+bool isHexDigit(const std::string& s)
+{
+  for(std::string::const_iterator i = s.begin(), eoi = s.end(); i != eoi; ++i) {
+    if(!isHexDigit(*i)) {
+      return false;
+    }
+  }
+  return true;
+}
+
 bool inRFC3986ReservedChars(const char c)
 {
   static const char reserved[] = {
@@ -211,16 +262,35 @@ bool inRFC3986ReservedChars(const char c)
 bool inRFC3986UnreservedChars(const char c)
 {
   static const char unreserved[] = { '-', '.', '_', '~' };
-  return
-    // ALPHA
-    ('A' <= c && c <= 'Z') || ('a' <= c && c <= 'z') ||
-    // DIGIT
-    ('0' <= c && c <= '9') ||
+  return isAlpha(c) || isDigit(c) ||
     std::find(&unreserved[0], &unreserved[arrayLength(unreserved)], c) !=
     &unreserved[arrayLength(unreserved)];
 }
 
-std::string urlencode(const unsigned char* target, size_t len) {
+bool inRFC2978MIMECharset(const char c)
+{
+  static const char chars[] = {
+    '!', '#', '$', '%', '&',
+    '\'', '+', '-', '^', '_',
+    '`', '{', '}', '~'
+  };
+  return isAlpha(c) || isDigit(c) ||
+    std::find(&chars[0], &chars[arrayLength(chars)], c) !=
+    &chars[arrayLength(chars)];
+}
+
+bool inRFC2616HttpToken(const char c)
+{
+  static const char chars[] = {
+    '!', '#', '$', '%', '&', '\'', '*', '+', '-', '.',
+    '^', '_', '`', '|', '~'
+  };
+  return isAlpha(c) || isDigit(c) ||
+    std::find(&chars[0], &chars[arrayLength(chars)], c) !=
+    &chars[arrayLength(chars)];
+}
+
+std::string percentEncode(const unsigned char* target, size_t len) {
   std::string dest;
   for(size_t i = 0; i < len; ++i) {
     if(!inRFC3986UnreservedChars(target[i])) {
@@ -232,18 +302,16 @@ std::string urlencode(const unsigned char* target, size_t len) {
   return dest;
 }
 
-std::string urlencode(const std::string& target)
+std::string percentEncode(const std::string& target)
 {
-  return urlencode(reinterpret_cast<const unsigned char*>(target.c_str()),
-		   target.size());
+  return percentEncode(reinterpret_cast<const unsigned char*>(target.c_str()),
+                   target.size());
 }
 
-std::string torrentUrlencode(const unsigned char* target, size_t len) {
+std::string torrentPercentEncode(const unsigned char* target, size_t len) {
   std::string dest;
   for(size_t i = 0; i < len; ++i) {
-    if(('0' <= target[i] && target[i] <= '9') ||
-       ('A' <= target[i] && target[i] <= 'Z') ||
-       ('a' <= target[i] && target[i] <= 'z')) {
+    if(isAlpha(target[i]) || isDigit(target[i])) {
       dest += target[i];
     } else {
       dest.append(StringFormat("%%%02X", target[i]).str());
@@ -252,23 +320,23 @@ std::string torrentUrlencode(const unsigned char* target, size_t len) {
   return dest;
 }
 
-std::string torrentUrlencode(const std::string& target)
+std::string torrentPercentEncode(const std::string& target)
 {
-  return torrentUrlencode
+  return torrentPercentEncode
     (reinterpret_cast<const unsigned char*>(target.c_str()), target.size());
 }
 
-std::string urldecode(const std::string& target) {
+std::string percentDecode(const std::string& target) {
   std::string result;
-  for(std::string::const_iterator itr = target.begin();
-      itr != target.end(); ++itr) {
+  for(std::string::const_iterator itr = target.begin(), eoi = target.end();
+      itr != eoi; ++itr) {
     if(*itr == '%') {
       if(itr+1 != target.end() && itr+2 != target.end() &&
-	 isxdigit(*(itr+1)) && isxdigit(*(itr+2))) {
-	result += parseInt(std::string(itr+1, itr+3), 16);
-	itr += 2;
+         isHexDigit(*(itr+1)) && isHexDigit(*(itr+2))) {
+        result += parseInt(std::string(itr+1, itr+3), 16);
+        itr += 2;
       } else {
-	result += *itr;
+        result += *itr;
       }
     } else {
       result += *itr;
@@ -278,14 +346,22 @@ std::string urldecode(const std::string& target) {
 }
 
 std::string toHex(const unsigned char* src, size_t len) {
-  char* temp = new char[len*2+1];
-  for(size_t i = 0; i < len; ++i) {
-    sprintf(temp+i*2, "%02x", src[i]);
+  std::string out(len*2, '\0');
+  std::string::iterator o = out.begin();
+  const unsigned char* last = src+len;
+  for(const unsigned char* i = src; i != last; ++i) {
+    *o = (*i >> 4);
+    *(o+1) = (*i)&0x0f;
+    for(int j = 0; j < 2; ++j) {
+      if(*o < 10) {
+        *o += '0';
+      } else {
+        *o += 'a'-10;
+      }
+      ++o;
+    }
   }
-  temp[len*2] = '\0';
-  std::string hex = temp;
-  delete [] temp;
-  return hex;
+  return out;
 }
 
 std::string toHex(const char* src, size_t len)
@@ -322,13 +398,13 @@ std::string fromHex(const std::string& src)
     return dest;
   }
   for(size_t i = 0; i < src.size(); i += 2) {
-      unsigned char high = hexCharToUInt(src[i]);
-      unsigned char low = hexCharToUInt(src[i+1]);
-      if(high == 255 || low == 255) {
-	dest.clear();
-	return dest;
-      }
-      dest += (high*16+low);
+    unsigned char high = hexCharToUInt(src[i]);
+    unsigned char low = hexCharToUInt(src[i+1]);
+    if(high == 255 || low == 255) {
+      dest.clear();
+      return dest;
+    }
+    dest += (high*16+low);
   }
   return dest;
 }
@@ -386,47 +462,20 @@ int getNum(const char* buf, int offset, size_t length) {
 
 int32_t parseInt(const std::string& s, int32_t base)
 {
-  std::string trimed = trim(s);
-  if(trimed.empty()) {
+  int64_t v = parseLLInt(s, base);
+  if(v < INT32_MIN || INT32_MAX < v) {
     throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-				 "empty string").str());
-  }
-  char* stop;
-  errno = 0;
-  long int v = strtol(trimed.c_str(), &stop, base);
-  if(*stop != '\0') {
-    throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-				 trimed.c_str()).str());
-  } else if((((v == LONG_MIN) || (v == LONG_MAX)) && (errno == ERANGE)) ||
-	    (v > INT32_MAX) ||
-	    (v < INT32_MIN)) {
-    throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-				 trimed.c_str()).str());
+                                   s.c_str()).str());
   }
   return v;
 }
 
 uint32_t parseUInt(const std::string& s, int base)
 {
-  std::string trimed = trim(s);
-  if(trimed.empty()) {
+  uint64_t v = parseULLInt(s, base);
+  if(UINT32_MAX < v) {
     throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-				 "empty string").str());
-  }
-  // We don't allow negative number.
-  if(trimed[0] == '-') {
-    throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-				 trimed.c_str()).str());
-  }
-  char* stop;
-  errno = 0;
-  unsigned long int v = strtoul(trimed.c_str(), &stop, base);
-  if(*stop != '\0') {
-    throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-				 trimed.c_str()).str());
-  } else if(((v == ULONG_MAX) && (errno == ERANGE)) || (v > UINT32_MAX)) {
-    throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-				 trimed.c_str()).str());
+                                   s.c_str()).str());
   }
   return v;
 }
@@ -458,17 +507,17 @@ int64_t parseLLInt(const std::string& s, int32_t base)
   std::string trimed = trim(s);
   if(trimed.empty()) {
     throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-				 "empty string").str());
+                                   "empty string").str());
   }
   char* stop;
   errno = 0;
   int64_t v = strtoll(trimed.c_str(), &stop, base);
   if(*stop != '\0') {
     throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-				 trimed.c_str()).str());
+                                   trimed.c_str()).str());
   } else if(((v == INT64_MIN) || (v == INT64_MAX)) && (errno == ERANGE)) {
     throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-				 trimed.c_str()).str());
+                                   trimed.c_str()).str());
   }
   return v;
 }
@@ -478,22 +527,22 @@ uint64_t parseULLInt(const std::string& s, int base)
   std::string trimed = trim(s);
   if(trimed.empty()) {
     throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-				 "empty string").str());
+                                   "empty string").str());
   }
   // We don't allow negative number.
   if(trimed[0] == '-') {
     throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-				 trimed.c_str()).str());
+                                   trimed.c_str()).str());
   }
   char* stop;
   errno = 0;
   uint64_t v = strtoull(trimed.c_str(), &stop, base);
   if(*stop != '\0') {
     throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-				 trimed.c_str()).str());
+                                   trimed.c_str()).str());
   } else if((v == ULLONG_MAX) && (errno == ERANGE)) {
     throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-				 trimed.c_str()).str());
+                                   trimed.c_str()).str());
   }
   return v;
 }
@@ -514,8 +563,8 @@ IntSequence parseIntRange(const std::string& src)
     } else {
       std::pair<std::string, std::string> vp = split(p.first.c_str(), "-");
       if(vp.first.empty() || vp.second.empty()) {
-	throw DL_ABORT_EX
-	  (StringFormat(MSG_INCOMPLETE_RANGE, p.first.c_str()).str());
+        throw DL_ABORT_EX
+          (StringFormat(MSG_INCOMPLETE_RANGE, p.first.c_str()).str());
       }
       int32_t v1 = parseInt(vp.first.c_str());
       int32_t v2 = parseInt(vp.second.c_str());
@@ -535,14 +584,14 @@ static void computeHeadPieces
     return;
   }
   for(std::vector<SharedHandle<FileEntry> >::const_iterator fi =
-	fileEntries.begin(); fi != fileEntries.end(); ++fi) {
+        fileEntries.begin(), eoi = fileEntries.end(); fi != eoi; ++fi) {
     if((*fi)->getLength() == 0) {
       continue;
     }
     size_t lastIndex =
       ((*fi)->getOffset()+std::min(head, (*fi)->getLength())-1)/pieceLength;
     for(size_t index = (*fi)->getOffset()/pieceLength;
-	index <= lastIndex; ++index) {
+        index <= lastIndex; ++index) {
       indexes.push_back(index);
     }
   }
@@ -558,7 +607,7 @@ static void computeTailPieces
     return;
   }
   for(std::vector<SharedHandle<FileEntry> >::const_iterator fi =
-	fileEntries.begin(); fi != fileEntries.end(); ++fi) {
+        fileEntries.begin(), eoi = fileEntries.end(); fi != eoi; ++fi) {
     if((*fi)->getLength() == 0) {
       continue;
     }
@@ -566,7 +615,7 @@ static void computeTailPieces
     size_t fromIndex =
       (endOffset-1-(std::min(tail, (*fi)->getLength())-1))/pieceLength;
     for(size_t index = fromIndex; index <= (endOffset-1)/pieceLength;
-	++index) {
+        ++index) {
       indexes.push_back(index);
     }
   }
@@ -575,31 +624,31 @@ static void computeTailPieces
 void parsePrioritizePieceRange
 (std::vector<size_t>& result, const std::string& src,
  const std::vector<SharedHandle<FileEntry> >& fileEntries,
- size_t pieceLength)
+ size_t pieceLength,
+ uint64_t defaultSize)
 {
-  const uint64_t DEFAULT_SIZE = 1024*1024;
   std::vector<size_t> indexes;
   std::vector<std::string> parts;
   split(src, std::back_inserter(parts), ",", true);
-  for(std::vector<std::string>::const_iterator i = parts.begin();
-      i != parts.end(); ++i) {
+  for(std::vector<std::string>::const_iterator i = parts.begin(),
+        eoi = parts.end(); i != eoi; ++i) {
     if((*i) == "head") {
-      computeHeadPieces(indexes, fileEntries, pieceLength, DEFAULT_SIZE);
+      computeHeadPieces(indexes, fileEntries, pieceLength, defaultSize);
     } else if(util::startsWith(*i, "head=")) {
       std::string sizestr = std::string((*i).begin()+(*i).find("=")+1,
-					(*i).end());
+                                        (*i).end());
       computeHeadPieces(indexes, fileEntries, pieceLength,
-			std::max((int64_t)0, getRealSize(sizestr)));
+                        std::max((int64_t)0, getRealSize(sizestr)));
     } else if((*i) == "tail") {
-      computeTailPieces(indexes, fileEntries, pieceLength, DEFAULT_SIZE);
+      computeTailPieces(indexes, fileEntries, pieceLength, defaultSize);
     } else if(util::startsWith(*i, "tail=")) {
       std::string sizestr = std::string((*i).begin()+(*i).find("=")+1,
-					(*i).end());
+                                        (*i).end());
       computeTailPieces(indexes, fileEntries, pieceLength,
-			std::max((int64_t)0, getRealSize(sizestr)));
+                        std::max((int64_t)0, getRealSize(sizestr)));
     } else {
       throw DL_ABORT_EX
-	(StringFormat("Unrecognized token %s", (*i).c_str()).str());
+        (StringFormat("Unrecognized token %s", (*i).c_str()).str());
     }
   }
   std::sort(indexes.begin(), indexes.end());
@@ -607,36 +656,140 @@ void parsePrioritizePieceRange
   result.insert(result.end(), indexes.begin(), indexes.end());
 }
 
-std::string getContentDispositionFilename(const std::string& header) {
-  static const std::string keyName = "filename=";
-  std::string::size_type attributesp = header.find(keyName);
-  if(attributesp == std::string::npos) {
-    return A2STR::NIL;
+// Converts ISO/IEC 8859-1 string to UTF-8 string.  If there is a
+// character not in ISO/IEC 8859-1, returns empty string.
+std::string iso8859ToUtf8(const std::string& src)
+{
+  std::string dest;
+  for(std::string::const_iterator itr = src.begin(), eoi = src.end();
+      itr != eoi; ++itr) {
+    unsigned char c = *itr;
+    if(0xa0 <= c) {
+      if(c <= 0xbf) {
+        dest += 0xc2;
+      } else {
+        dest += 0xc3;
+      }
+      dest += c&(~0x40);
+    } else if(0x80 <= c && c <= 0x9f) {
+      return A2STR::NIL;
+    } else {
+      dest += c;
+    }
   }
-  std::string::size_type filenamesp = attributesp+keyName.size();
-  std::string::size_type filenameep;
-  if(filenamesp == header.size()) {
-    return A2STR::NIL;
-  }
-  
-  if(header[filenamesp] == '\'' || header[filenamesp] == '"') {
-    char quoteChar = header[filenamesp];
-    filenameep = header.find(quoteChar, filenamesp+1);
-  } else {
-    filenameep = header.find(';', filenamesp);
-  }
-  if(filenameep == std::string::npos) {
-    filenameep = header.size();
-  }
-  static const std::string TRIMMED("\r\n '\"");
-  std::string fn =
-    File(trim(header.substr
-	      (filenamesp, filenameep-filenamesp), TRIMMED)).getBasename();
-  if(fn == ".." || fn == ".") {
-    return A2STR::NIL;
-  } else {
-    return fn;
+  return dest;
+}
+
+std::string getContentDispositionFilename(const std::string& header)
+{
+  std::string filename;
+  std::vector<std::string> params;
+  split(header, std::back_inserter(params), A2STR::SEMICOLON_C, true);
+  for(std::vector<std::string>::const_iterator i = params.begin(),
+        eoi = params.end(); i != eoi; ++i) {
+    const std::string& param = *i;
+    static const std::string keyName = "filename";
+    if(!startsWith(toLower(param), keyName) || param.size() == keyName.size()) {
+      continue;
+    }
+    std::string::const_iterator markeritr = param.begin()+keyName.size();
+    if(*markeritr == '*') {
+      // See RFC2231 Section4 and draft-reschke-rfc2231-in-http.
+      // Please note that this function doesn't do charset conversion
+      // except that if iso-8859-1 is specified, it is converted to
+      // utf-8.
+      ++markeritr;
+      for(; markeritr != param.end() && *markeritr == ' '; ++markeritr);
+      if(markeritr == param.end() || *markeritr != '=') {
+        continue;
+      }
+      std::pair<std::string, std::string> paramPair;
+      split(paramPair, param, '=');
+      std::string value = paramPair.second;
+      std::vector<std::string> extValues;
+      split(value, std::back_inserter(extValues), "'", false, true);
+      if(extValues.size() != 3) {
+        continue;
+      }
+      bool bad = false;
+      const std::string& charset = extValues[0];
+      for(std::string::const_iterator j = charset.begin(), eoi = charset.end();
+          j != eoi; ++j) {
+        // Since we first split parameter by ', we can safely assume
+        // that ' is not included in charset.
+        if(!inRFC2978MIMECharset(*j)) {
+          bad = true;
+          break;
+        }
+      }
+      if(bad) {
+        continue;
+      }
+      bad = false;
+      value = extValues[2];
+      for(std::string::const_iterator j = value.begin(), eoi = value.end();
+          j != eoi; ++j){
+        if(*j == '%') {
+          if(j+1 != value.end() && isHexDigit(*(j+1)) &&
+             j+2 != value.end() && isHexDigit(*(j+2))) {
+            j += 2;
+          } else {
+            bad = true;
+            break;
+          }
+        } else {
+          if(*j == '*' || *j == '\'' || !inRFC2616HttpToken(*j)) {
+            bad = true;
+            break;
+          }
+        }
+      }
+      if(bad) {
+        continue;
+      }
+      value = percentDecode(value);
+      if(toLower(extValues[0]) == "iso-8859-1") {
+        value = iso8859ToUtf8(value);
+      }
+      if(!detectDirTraversal(value) &&
+         value.find(A2STR::SLASH_C) == std::string::npos) {
+        filename = value;
+      }
+      if(!filename.empty()) {
+        break;
+      }
+    } else {
+      for(; markeritr != param.end() && *markeritr == ' '; ++markeritr);
+      if(markeritr == param.end() || *markeritr != '=') {
+        continue;
+      }
+      std::pair<std::string, std::string> paramPair;
+      split(paramPair, param, '=');
+      std::string value = paramPair.second;
+      if(value.empty()) {
+        continue;
+      }
+      std::string::iterator filenameLast;
+      if(*value.begin() == '\'' || *value.begin() == '"') {
+        char qc = *value.begin();
+        for(filenameLast = value.begin()+1;
+            filenameLast != value.end() && *filenameLast != qc;
+            ++filenameLast);
+      } else {
+        filenameLast = value.end();
+      }
+      static const std::string TRIMMED("\r\n\t '\"");
+      value = percentDecode(std::string(value.begin(), filenameLast));
+      trimSelf(value, TRIMMED);
+      value.erase(std::remove(value.begin(), value.end(), '\\'), value.end());
+      if(!detectDirTraversal(value) &&
+         value.find(A2STR::SLASH_C) == std::string::npos) {
+        filename = value;
+      }
+      // continue because there is a chance we can find filename*=...
+    }
   }
+  return filename;
 }
 
 std::string randomAlpha(size_t length, const RandomizerHandle& randomizer) {
@@ -661,13 +814,18 @@ std::string toLower(const std::string& src) {
   return temp;
 }
 
-bool isNumbersAndDotsNotation(const std::string& name) {
-  struct sockaddr_in sockaddr;
-  if(inet_aton(name.c_str(), &sockaddr.sin_addr)) {
-    return true;
-  } else {
+bool isNumericHost(const std::string& name)
+{
+  struct addrinfo hints;
+  struct addrinfo* res;
+  memset(&hints, 0, sizeof(hints));
+  hints.ai_family = AF_UNSPEC;
+  hints.ai_flags = AI_NUMERICHOST;
+  if(getaddrinfo(name.c_str(), 0, &hints, &res)) {
     return false;
   }
+  freeaddrinfo(res);
+  return true;
 }
 
 void setGlobalSignalHandler(int sig, void (*handler)(int), int flags) {
@@ -714,7 +872,7 @@ int64_t getRealSize(const std::string& sizeWithUnit)
       (StringFormat("Negative value detected: %s", sizeWithUnit.c_str()).str());
   } else if(INT64_MAX/mult < v) {
     throw DL_ABORT_EX(StringFormat(MSG_STRING_INTEGER_CONVERSION_FAILURE,
-				 "overflow/underflow").str());
+                                   "overflow/underflow").str());
   }
   return v*mult;
 }
@@ -734,7 +892,7 @@ std::string abbrevSize(int64_t size)
     size >>= 10;
   }
   std::string result = itos(size, true);
-  result += ".";
+  result += A2STR::DOT_C;
   result += itos(r*10/1024);
   result += units[i];
   result += "i";
@@ -758,33 +916,33 @@ void usleep(long microseconds) {
   ::usleep(microseconds);
 #elif defined(HAVE_WINSOCK2_H)
 
-	LARGE_INTEGER current, freq, end;
+  LARGE_INTEGER current, freq, end;
 
-	static enum {GET_FREQUENCY, GET_MICROSECONDS, SKIP_MICROSECONDS} state = GET_FREQUENCY;
+  static enum {GET_FREQUENCY, GET_MICROSECONDS, SKIP_MICROSECONDS} state = GET_FREQUENCY;
 
-	if (state == GET_FREQUENCY) {
-		if (QueryPerformanceFrequency(&freq))
-			state = GET_MICROSECONDS;
-		else
-			state = SKIP_MICROSECONDS;
-	}
-	
-	long msec = microseconds / 1000;
-	microseconds %= 1000;    
+  if (state == GET_FREQUENCY) {
+    if (QueryPerformanceFrequency(&freq))
+      state = GET_MICROSECONDS;
+    else
+      state = SKIP_MICROSECONDS;
+  }
+        
+  long msec = microseconds / 1000;
+  microseconds %= 1000;    
 
-	if (state == GET_MICROSECONDS && microseconds) {
-		QueryPerformanceCounter(&end);
+  if (state == GET_MICROSECONDS && microseconds) {
+    QueryPerformanceCounter(&end);
 
-		end.QuadPart += (freq.QuadPart * microseconds) / 1000000;
+    end.QuadPart += (freq.QuadPart * microseconds) / 1000000;
 
-		while (QueryPerformanceCounter(&current) && (current.QuadPart <= end.QuadPart))
-			/* noop */ ;
-	}
+    while (QueryPerformanceCounter(&current) && (current.QuadPart <= end.QuadPart))
+      /* noop */ ;
+  }
 
-	if (msec)
-		Sleep(msec);
+  if (msec)
+    Sleep(msec);
 #else
-	#error no usleep function is available (nanosleep?)
+#error no usleep function is available (nanosleep?)
 #endif
 }
 
@@ -793,8 +951,9 @@ bool isNumber(const std::string& what)
   if(what.empty()) {
     return false;
   }
-  for(uint32_t i = 0; i < what.size(); ++i) {
-    if(!isdigit(what[i])) {
+  for(std::string::const_iterator i = what.begin(), eoi = what.end();
+      i != eoi; ++i) {
+    if(!isDigit(*i)) {
       return false;
     }
   }
@@ -857,11 +1016,11 @@ void mkdirs(const std::string& dirpath)
   } else if(dir.exists()) {
     throw DL_ABORT_EX
       (StringFormat(EX_MAKE_DIR, dir.getPath().c_str(),
-		    "File already exists.").str());
+                    "File already exists.").str());
   } else if(!dir.mkdirs()) {
     throw DL_ABORT_EX
       (StringFormat(EX_MAKE_DIR, dir.getPath().c_str(),
-		    strerror(errno)).str());
+                    strerror(errno)).str());
   }
 }
 
@@ -870,7 +1029,7 @@ void convertBitfield(BitfieldMan* dest, const BitfieldMan* src)
   size_t numBlock = dest->countBlock();
   for(size_t index = 0; index < numBlock; ++index) {
     if(src->isBitSetOffsetRange((uint64_t)index*dest->getBlockLength(),
-				dest->getBlockLength())) {
+                                dest->getBlockLength())) {
       dest->setBit(index);
     }
   }
@@ -913,10 +1072,10 @@ getNumericNameInfo(const struct sockaddr* sockaddr, socklen_t len)
   char host[NI_MAXHOST];
   char service[NI_MAXSERV];
   int s = getnameinfo(sockaddr, len, host, NI_MAXHOST, service, NI_MAXSERV,
-		      NI_NUMERICHOST|NI_NUMERICSERV);
+                      NI_NUMERICHOST|NI_NUMERICSERV);
   if(s != 0) {
     throw DL_ABORT_EX(StringFormat("Failed to get hostname and port. cause: %s",
-				 gai_strerror(s)).str());
+                                   gai_strerror(s)).str());
   }
   return std::pair<std::string, uint16_t>(host, atoi(service)); // TODO
 }
@@ -924,7 +1083,8 @@ getNumericNameInfo(const struct sockaddr* sockaddr, socklen_t len)
 std::string htmlEscape(const std::string& src)
 {
   std::string dest;
-  for(std::string::const_iterator i = src.begin(); i != src.end(); ++i) {
+  for(std::string::const_iterator i = src.begin(), eoi = src.end();
+      i != eoi; ++i) {
     char ch = *i;
     if(ch == '<') {
       dest += "&lt;";
@@ -950,7 +1110,7 @@ parseIndexPath(const std::string& line)
   size_t index = parseUInt(p.first);
   if(p.second.empty()) {
     throw DL_ABORT_EX(StringFormat("Path with index=%u is empty.",
-				 static_cast<unsigned int>(index)).str());
+                                   static_cast<unsigned int>(index)).str());
   }
   return std::map<size_t, std::string>::value_type(index, p.second);
 }
@@ -981,6 +1141,128 @@ void generateRandomData(unsigned char* data, size_t length)
 #endif // HAVE_LIBSSL
 }
 
+bool saveAs
+(const std::string& filename, const std::string& data, bool overwrite)
+{
+  if(!overwrite && File(filename).exists()) {
+    return false;
+  }
+  std::string tempFilename = strconcat(filename, "__temp");
+  std::ofstream out(tempFilename.c_str(), std::ios::binary);
+  if(!out) {
+    return false;
+  }
+  out << data;
+  out.flush();
+  if(!out) {
+    return false;
+  }
+  return File(tempFilename).renameTo(filename);
+}
+
+std::string applyDir(const std::string& dir, const std::string& relPath)
+{
+  if(dir.empty()) {
+    return strconcat(A2STR::DOT_C, A2STR::SLASH_C, relPath);
+  } else if(dir == A2STR::SLASH_C) {
+    return strconcat(A2STR::SLASH_C, relPath);
+  } else {
+    return strconcat(dir, A2STR::SLASH_C, relPath);
+  }
+}
+
+std::string fixTaintedBasename(const std::string& src)
+{
+  return escapePath(replace(src, A2STR::SLASH_C, A2STR::UNDERSCORE_C));
+}
+
+void generateRandomKey(unsigned char* key)
+{
+#ifdef ENABLE_MESSAGE_DIGEST
+  unsigned char bytes[40];
+  generateRandomData(bytes, sizeof(bytes));
+  MessageDigestHelper::digest
+    (key, 20, MessageDigestContext::SHA1, bytes, sizeof(bytes));
+#else // !ENABLE_MESSAGE_DIGEST
+  generateRandomData(key, 20);
+#endif // !ENABLE_MESSAGE_DIGEST
+}
+
+// Returns true is given numeric ipv4addr is in Private Address Space.
+//
+// From Section.3 RFC1918
+// 10.0.0.0        -   10.255.255.255  (10/8 prefix)
+// 172.16.0.0      -   172.31.255.255  (172.16/12 prefix)
+// 192.168.0.0     -   192.168.255.255 (192.168/16 prefix)
+bool inPrivateAddress(const std::string& ipv4addr)
+{
+  if(util::startsWith(ipv4addr, "10.") ||
+     util::startsWith(ipv4addr, "192.168.")) {
+    return true;
+  }
+  if(util::startsWith(ipv4addr, "172.")) {
+    for(int i = 16; i <= 31; ++i) {
+      if(util::startsWith(ipv4addr, "172."+util::itos(i)+".")) {
+        return true;
+      }
+    }
+  }
+  return false;
+}
+
+bool detectDirTraversal(const std::string& s)
+{
+  for(std::string::const_iterator i = s.begin(), eoi = s.end(); i != eoi; ++i) {
+    if(0x00 <= (*i) && (*i) <= 0x1f) {
+      return true;
+    }
+  }
+  return s == A2STR::DOT_C ||
+    s == ".." ||
+    util::startsWith(s, A2STR::SLASH_C) ||
+    util::startsWith(s, "./") ||
+    util::startsWith(s, "../") ||
+    s.find("/../") != std::string::npos ||
+    s.find("/./") != std::string::npos ||
+    util::endsWith(s, "/") ||
+    util::endsWith(s, "/.") ||
+    util::endsWith(s, "/..");
+}
+
+namespace {
+class EscapePath {
+private:
+  char _repChar;
+public:
+  EscapePath(const char& repChar):_repChar(repChar) {}
+
+  char operator()(const char& c) {
+    if(0x00 <= c && c <=0x1f) {
+      return _repChar;
+    }
+#ifdef __MINGW32__
+    // We don't escape '/' because we use it as a path separator.
+    static const char WIN_INVALID_PATH_CHARS[] =
+      { '"', '*', ':', '<', '>', '?', '\\', '|' };
+    if(std::find(&WIN_INVALID_PATH_CHARS[0],
+                 &WIN_INVALID_PATH_CHARS[arrayLength(WIN_INVALID_PATH_CHARS)],
+                 c) !=
+       &WIN_INVALID_PATH_CHARS[arrayLength(WIN_INVALID_PATH_CHARS)]) {
+      return _repChar;
+    }
+#endif // __MINGW32__
+    return c;
+  }
+};
+}
+
+std::string escapePath(const std::string& s)
+{
+  std::string d = s;
+  std::transform(d.begin(), d.end(), d.begin(), EscapePath('_'));
+  return d;
+}
+
 } // namespace util
 
 } // namespace aria2
