@@ -3,7 +3,7 @@
 #include <cppunit/extensions/HelperMacros.h>
 
 #include "Exception.h"
-#include "Util.h"
+#include "util.h"
 #include "array_fun.h"
 #include "FileEntry.h"
 
@@ -27,7 +27,7 @@ public:
       "ftp://alpha/file",
       "http://bravo/file"
     };
-    std::deque<std::string> uris;
+    std::vector<std::string> uris;
     uris.assign(&urisSrc[0], &urisSrc[arrayLength(urisSrc)]);
     
     _fileEntry.setUris(uris);
@@ -46,11 +46,11 @@ CPPUNIT_TEST_SUITE_REGISTRATION(InOrderURISelectorTest);
 void InOrderURISelectorTest::testSelect()
 {
   CPPUNIT_ASSERT_EQUAL(std::string("http://alpha/file"),
-		       sel->select(&_fileEntry));
+                       sel->select(&_fileEntry));
   CPPUNIT_ASSERT_EQUAL(std::string("ftp://alpha/file"),
-		       sel->select(&_fileEntry));
+                       sel->select(&_fileEntry));
   CPPUNIT_ASSERT_EQUAL(std::string("http://bravo/file"),
-		       sel->select(&_fileEntry));
+                       sel->select(&_fileEntry));
   CPPUNIT_ASSERT_EQUAL(std::string(""), sel->select(&_fileEntry));
 }
 
